@@ -863,13 +863,20 @@ void page_tree(void){
   }else{
     Stmt q;
     db_prepare(&q,
+      "WITH mx(fnid,fid,mtime) AS (\n"
+      "  SELECT fnid, fid, max(event.mtime)\n"
+      "    FROM mlink, event\n"
+      "   WHERE event.objid=mlink.mid\n"
+      "   GROUP BY 1\n"
+      ")\n"
       "SELECT\n"
-      "    (SELECT name FROM filename WHERE filename.fnid=mlink.fnid),\n"
-      "    (SELECT uuid FROM blob WHERE blob.rid=mlink.fid),\n"
-      "    (SELECT size FROM blob WHERE blob.rid=mlink.fid),\n"
-      "    max(event.mtime)\n"
-      "  FROM mlink JOIN event ON event.objid=mlink.mid\n"
-      " GROUP BY mlink.fnid\n"
+      "  filename.name,\n"
+      "  blob.uuid,\n"
+      "  blob.size,\n"
+      "  mx.mtime\n"
+      "FROM mx\n"
+      " LEFT JOIN filename ON filename.fnid=mx.fnid\n"
+      " LEFT JOIN blob ON blob.rid=mx.fid\n"
       " ORDER BY 1 COLLATE uintnocase;");
     while( db_step(&q)==SQLITE_ROW ){
       const char *zName = db_column_text(&q, 0);
@@ -1158,7 +1165,7 @@ void fileage_page(void){
   double baseTime;
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  if( exclude_spiders() ) return;
+  if( exclude_spiders(0) ) return;
   zName = P("name");
   if( zName==0 ) zName = "tip";
   rid = symbolic_name_to_rid(zName, "ci");
@@ -1254,20 +1261,3 @@ void fileage_page(void){
   db_finalize(&q2);
   style_finish_page();
 }
-
-/*
-** WEBPAGE: files
-**
-** Show files as a flat table.  If the ci=LABEL query parameter is provided,
-** then show all the files in the specified check-in.  Without the ci= query
-** parameter show all files across all check-ins.
-**
-** Query parameters:
-**
-**    name=PATH        Directory to display.  Optional
-**    ci=LABEL         Show only files in this check-in.  Optional.
-**    re=REGEXP        Show only files matching REGEXP.  Optional.
-*/
-void files_page(void){
-  return;
-}
