@@ -25,7 +25,7 @@
 #include "captcha.h"
 
 #if INTERFACE
-#define CAPTCHA 3  /* Which captcha rendering to use */
+#define CAPTCHA 2  /* Which captcha rendering to use */
 #endif
 
 /*
@@ -72,7 +72,7 @@ static const unsigned int aFont1[] = {
 ** by the caller.
 */
 char *captcha_render(const char *zPw){
-  char *z = fossil_malloc( 9*6*strlen(zPw) + 7 );
+  char *z = fossil_malloc( 9*12*3*strlen(zPw) + 8 );
   int i, j, k, m;
 
   k = 0;
@@ -82,8 +82,12 @@ char *captcha_render(const char *zPw){
       v = (aFont1[v] >> ((5-i)*4)) & 0xf;
       for(m=8; m>=1; m = m>>1){
         if( v & m ){
-          z[k++] = 'X';
-          z[k++] = 'X';
+          z[k++] = 0xe2;
+          z[k++] = 0x96;
+          z[k++] = 0x88;
+          z[k++] = 0xe2;
+          z[k++] = 0x96;
+          z[k++] = 0x88;
         }else{
           z[k++] = ' ';
           z[k++] = ' ';
@@ -99,9 +103,69 @@ char *captcha_render(const char *zPw){
 }
 #endif /* CAPTCHA==1 */
 
-
 #if CAPTCHA==2
-static const char *const azFont2[] = {
+/*
+** A 5x7 pixel bitmap font for hexadecimal digits
+*/
+static const unsigned char aFont2[] = {
+ /* 0 */  0x0e, 0x13, 0x15, 0x19, 0x11, 0x11, 0x0e,
+ /* 1 */  0x02, 0x06, 0x0A, 0x02, 0x02, 0x02, 0x02,
+ /* 2 */  0x0e, 0x11, 0x01, 0x02, 0x04, 0x08, 0x1f,
+ /* 3 */  0x0e, 0x11, 0x01, 0x06, 0x01, 0x11, 0x0e,
+ /* 4 */  0x02, 0x06, 0x0A, 0x12, 0x1f, 0x02, 0x02,
+ /* 5 */  0x1f, 0x10, 0x1e, 0x01, 0x01, 0x11, 0x0e,
+ /* 6 */  0x0e, 0x11, 0x10, 0x1e, 0x11, 0x11, 0x0e,
+ /* 7 */  0x1f, 0x01, 0x02, 0x04, 0x08, 0x08, 0x08,
+ /* 8 */  0x0e, 0x11, 0x11, 0x0e, 0x11, 0x11, 0x0e,
+ /* 9 */  0x0e, 0x11, 0x11, 0x0f, 0x01, 0x11, 0x0e,
+ /* A */  0x0e, 0x11, 0x11, 0x11, 0x1f, 0x11, 0x11,
+ /* B */  0x1e, 0x11, 0x11, 0x1e, 0x11, 0x11, 0x1e,
+ /* C */  0x0e, 0x11, 0x10, 0x10, 0x10, 0x11, 0x0e,
+ /* D */  0x1c, 0x12, 0x11, 0x11, 0x11, 0x12, 0x1c,
+ /* E */  0x1f, 0x10, 0x10, 0x1c, 0x10, 0x10, 0x1f,
+ /* F */  0x1f, 0x10, 0x10, 0x1e, 0x10, 0x10, 0x10,
+};
+
+/*
+** Render an 8-character hexadecimal string as ascii art.
+** Space to hold the result is obtained from malloc() and should be freed
+** by the caller.
+*/
+char *captcha_render(const char *zPw){
+  char *z = fossil_malloc( 160*strlen(zPw) + 9 );
+  int i, j, k, m;
+
+  k = 0;
+  for(i=0; i<7; i++){
+    for(j=0; zPw[j]; j++){
+      unsigned char v = hex_digit_value(zPw[j]);
+      v = aFont2[v*7+i];
+      for(m=16; m>=1; m = m>>1){
+        if( v & m ){
+          z[k++] = 0xe2;
+          z[k++] = 0x96;
+          z[k++] = 0x88;
+          z[k++] = 0xe2;
+          z[k++] = 0x96;
+          z[k++] = 0x88;
+        }else{
+          z[k++] = ' ';
+          z[k++] = ' ';
+        }
+      }
+      z[k++] = ' ';
+      z[k++] = ' ';
+    }
+    z[k++] = '\n';
+  }
+  z[k] = 0;
+  return z;
+}
+#endif /* CAPTCHA==2 */
+
+
+#if CAPTCHA==3
+static const char *const azFont3[] = {
  /* 0 */
  "  __  ",
  " /  \\ ",
@@ -213,7 +277,7 @@ char *captcha_render(const char *zPw){
   for(i=0; i<4; i++){
     for(j=0; zPw[j]; j++){
       unsigned char v = hex_digit_value(zPw[j]);
-      zChar = azFont2[4*v + i];
+      zChar = azFont3[4*v + i];
       for(m=0; zChar[m]; m++){
         z[k++] = zChar[m];
       }
@@ -223,10 +287,10 @@ char *captcha_render(const char *zPw){
   z[k] = 0;
   return z;
 }
-#endif /* CAPTCHA==2 */
+#endif /* CAPTCHA==3 */
 
-#if CAPTCHA==3
-static const char *const azFont3[] = {
+#if CAPTCHA==4
+static const char *const azFont4[] = {
   /* 0 */
   "  ___  ",
   " / _ \\ ",
@@ -401,7 +465,7 @@ char *captcha_render(const char *zPw){
           y = 0;
           break;
       }
-      zChar = azFont3[6*v + i];
+      zChar = azFont4[6*v + i];
       while( y && zChar[0]==' ' ){ y--; zChar++; }
       while( y && z[k-1]==' ' ){ y--; k--; }
       for(m=0; zChar[m]; m++){
@@ -413,7 +477,7 @@ char *captcha_render(const char *zPw){
   z[k] = 0;
   return z;
 }
-#endif /* CAPTCHA==3 */
+#endif /* CAPTCHA==4 */
 
 /*
 ** COMMAND: test-captcha
@@ -448,26 +512,108 @@ unsigned int captcha_seed(void){
   return x;
 }
 
+/* The SQL that will rotate the the captcha-secret. */
+static const char captchaSecretRotationSql[] = 
+"SAVEPOINT rotate;\n"
+"DELETE FROM config\n"
+" WHERE name GLOB 'captcha-secret-*'\n"
+"   AND mtime<unixepoch('now','-6 hours');\n"
+"UPDATE config\n"
+"   SET name=format('captcha-secret-%%d',substr(name,16)+1)\n"
+" WHERE name GLOB 'captcha-secret-*';\n"
+"UPDATE config\n"
+"   SET name='captcha-secret-1', mtime=unixepoch()\n"
+" WHERE name='captcha-secret';\n"
+"REPLACE INTO config(name,value,mtime)\n"
+"  VALUES('captcha-secret',%Q,unixepoch());\n"
+"RELEASE rotate;\n"
+;
+
+
+/*
+** Create a new random captcha-secret.  Rotate the old one into
+** the captcha-secret-N backups.  Purge captch-secret-N backups
+** older than 6 hours.
+**
+** Do this on the current database and in all other databases of
+** the same login group.
+*/
+void captcha_secret_rotate(void){
+  char *zNew = db_text(0, "SELECT lower(hex(randomblob(20)))");
+  char *zSql = mprintf(captchaSecretRotationSql/*works-like:"%Q"*/, zNew);
+  char *zErrs = 0;
+  fossil_free(zNew);
+  db_unprotect(PROTECT_CONFIG);
+  db_begin_transaction();
+  sqlite3_exec(g.db, zSql, 0, 0, &zErrs);
+  db_protect_pop();
+  if( zErrs && zErrs[0] ){
+    db_rollback_transaction();
+    fossil_fatal("Unable to rotate captcha-secret\n%s\nERROR: %s\n",
+                 zSql, zErrs);
+  }
+  db_end_transaction(0);
+  login_group_sql(zSql, "", "", &zErrs);
+  if( zErrs ){
+    sqlite3_free(zErrs);  /* Silently ignore errors on other repos */
+  }
+  fossil_free(zSql);
+}
+
+/*
+** Return the value of the N-th more recent captcha-secret.  The
+** most recent captch-secret is 0.  Others are prior captcha-secrets
+** that have expired, but are retained for a limited period of time
+** so that pending anonymous login cookies and/or captcha dialogs
+** don't malfunction when the captcha-secret changes.
+**
+** Clients should start by using the 0-th captcha-secret.  Only if
+** that one does not work should they advance to 1 and 2 and so forth,
+** until this routine returns a NULL pointer.
+**
+** The value returned is a string obtained from fossil_malloc() and
+** should be freed by the caller.
+**
+** The 0-th captcha secret is the value of Config.Name='captcha-secret'.
+** For N>0, the value is in Config.Name='captcha-secret-$N'.
+*/
+char *captcha_secret(int N){
+  if( N==0 ){
+    return db_text(0, "SELECT value FROM config WHERE name='captcha-secret'");
+  }else{
+    return db_text(0, 
+        "SELECT value FROM config"
+        " WHERE name='captcha-secret-%d'"
+        "   AND mtime>unixepoch('now','-6 hours')", N);
+  }
+}
+
 /*
 ** Translate a captcha seed value into the captcha password string.
 ** The returned string is static and overwritten on each call to
 ** this function.
+**
+** Use the N-th captcha secret to compute the password.  When N==0,
+** a valid password is always returned.  A new captcha-secret will
+** be created if necessary.  But for N>0, the return value might
+** be NULL to indicate that there is no N-th captcha-secret.
 */
-const char *captcha_decode(unsigned int seed){
-  const char *zSecret;
+const char *captcha_decode(unsigned int seed, int N){
+  char *zSecret;
   const char *z;
   Blob b;
   static char zRes[20];
 
-  zSecret = db_get("captcha-secret", 0);
+  zSecret = captcha_secret(N);
   if( zSecret==0 ){
+    if( N>0 ) return 0;
     db_unprotect(PROTECT_CONFIG);
     db_multi_exec(
       "REPLACE INTO config(name,value)"
       " VALUES('captcha-secret', lower(hex(randomblob(20))));"
     );
     db_protect_pop();
-    zSecret = db_get("captcha-secret", 0);
+    zSecret = captcha_secret(0);
     assert( zSecret!=0 );
   }
   blob_init(&b, 0, 0);
@@ -476,6 +622,7 @@ const char *captcha_decode(unsigned int seed){
   z = blob_buffer(&b);
   memcpy(zRes, z, 8);
   zRes[8] = 0;
+  fossil_free(zSecret);
   return zRes;
 }
 
@@ -508,6 +655,7 @@ int captcha_is_correct(int bAlwaysNeeded){
   const char *zDecode;
   char z[30];
   int i;
+  int n = 0;
   if( !bAlwaysNeeded && !captcha_needed() ){
     return 1;  /* No captcha needed */
   }
@@ -515,15 +663,17 @@ int captcha_is_correct(int bAlwaysNeeded){
   if( zSeed==0 ) return 0;
   zEntered = P("captcha");
   if( zEntered==0 || strlen(zEntered)!=8 ) return 0;
-  zDecode = captcha_decode((unsigned int)atoi(zSeed));
-  assert( strlen(zDecode)==8 );
-  for(i=0; i<8; i++){
-    char c = zEntered[i];
-    if( c>='A' && c<='F' ) c += 'a' - 'A';
-    if( c=='O' ) c = '0';
-    z[i] = c;
-  }
-  if( strncmp(zDecode,z,8)!=0 ) return 0;
+  do{
+    zDecode = captcha_decode((unsigned int)atoi(zSeed), n++);
+    if( zDecode==0 ) return 0;
+    assert( strlen(zDecode)==8 );
+    for(i=0; i<8; i++){
+      char c = zEntered[i];
+      if( c>='A' && c<='F' ) c += 'a' - 'A';
+      if( c=='O' ) c = '0';
+      z[i] = c;
+    }
+  }while( strncmp(zDecode,z,8)!=0 );
   return 1;
 }
 
@@ -533,23 +683,28 @@ int captcha_is_correct(int bAlwaysNeeded){
 ** the captcha.  This is typically done at the very bottom of a form.
 **
 ** This routine is a no-op if no captcha is required.
+**
+** Flag values:
+**
+**     0x01     Show the "Submit" button in the form.
+**     0x02     Always generate the captcha, even if not required
 */
-void captcha_generate(int showButton){
+void captcha_generate(int mFlags){
   unsigned int uSeed;
   const char *zDecoded;
   char *zCaptcha;
 
-  if( !captcha_needed() ) return;
+  if( !captcha_needed() && (mFlags & 0x02)==0 ) return;
   uSeed = captcha_seed();
-  zDecoded = captcha_decode(uSeed);
+  zDecoded = captcha_decode(uSeed, 0);
   zCaptcha = captcha_render(zDecoded);
   cgi_printf("<div class=\"captcha\"><table class=\"captcha\"><tr><td><pre class=\"captcha\">\n"
          "%h\n"
          "</pre>\n"
          "Enter security code shown above:\n"
          "<input type=\"hidden\" name=\"captchaseed\" value=\"%u\">\n"
-         "<input type=\"text\" name=\"captcha\" size=8>\n",(zCaptcha),(uSeed));
-  if( showButton ){
+         "<input type=\"text\" name=\"captcha\" size=\"8\" autofocus>\n",(zCaptcha),(uSeed));
+  if( mFlags & 0x01 ){
     cgi_printf("<input type=\"submit\" value=\"Submit\">\n");
   }
   cgi_printf("<br/>");
@@ -577,64 +732,117 @@ void captcha_speakit_button(unsigned int uSeed, const char *zMsg){
 
 /*
 ** WEBPAGE: test-captcha
-** Test the captcha-generator by rendering the value of the name= query
-** parameter using ascii-art.  If name= is omitted, show a random 16-digit
-** hexadecimal number.
+**
+** If the name query parameter is provided, then render the hex value of
+** the name using the captcha font.
+**
+** Otherwise render the captcha screen.  The "show-button" parameter causes
+** the submit button to be rendered.
 */
 void captcha_test(void){
   const char *zPw = P("name");
   if( zPw==0 || zPw[0]==0 ){
-    u64 x;
-    sqlite3_randomness(sizeof(x), &x);
-    zPw = mprintf("%016llx", x);
+    (void)exclude_spiders(1);
+    cgi_printf("<hr><p>The captcha is shown above.  Add a name=HEX query parameter\n"
+           "to see how HEX would be rendered in the current captcha font.\n"
+           "<h2>Debug/Testing Values:</h2>\n"
+           "<ul>\n"
+           "<li> g.isHuman = %d\n"
+           "<li> g.zLogin = %h\n"
+           "<li> login_cookie_welformed() = %d\n"
+           "<li> captcha_is_correct(1) = %d.\n"
+           "</ul>\n",(g.isHuman),(g.zLogin),(login_cookie_wellformed()),(captcha_is_correct(1)));
+    style_finish_page();
+  }else{
+    style_set_current_feature("test");
+    style_header("Captcha Test");
+    cgi_printf("<pre class=\"captcha\">\n"
+           "%s\n"
+           "</pre>\n",(captcha_render(zPw)));
+    style_finish_page();
   }
-  style_set_current_feature("test");
-  style_header("Captcha Test");
-  cgi_printf("<pre>\n"
-         "%s\n"
-         "</pre>\n",(captcha_render(zPw)));
-  style_finish_page();
 }
 
 /*
-** Check to see if the current request is coming from an agent that might
-** be a spider.  If the agent is not a spider, then return 0 without doing
-** anything.  But if the user agent appears to be a spider, offer
-** a captcha challenge to allow the user agent to prove that it is human
-** and return non-zero.
+** Check to see if the current request is coming from an agent that 
+** self-identifies as a spider.
+**
+** If the agent does not claim to be a spider or if the user has logged
+** in (even as anonymous), then return 0 without doing anything.
+**
+** But if the user agent does self-identify as a spider and there is
+** no login, offer a captcha challenge to allow the user agent to prove
+** that he is human and return non-zero.
+**
+** If the bTest argument is non-zero, then show the captcha regardless of
+** how the agent identifies.  This is used for testing only.
 */
-int exclude_spiders(void){
-  const char *zCookieValue;
-  char *zCookieName;
-  if( g.isHuman ) return 0;
-#if 0
-  {
-    const char *zReferer = P("HTTP_REFERER");
-    if( zReferer && strncmp(g.zBaseURL, zReferer, strlen(g.zBaseURL))==0 ){
+int exclude_spiders(int bTest){
+  if( !bTest ){
+    if( g.isHuman ) return 0;  /* This user has already proven human */
+    if( g.zLogin!=0 ) return 0;  /* Logged in.  Consider them human */
+    if( login_cookie_wellformed() ){
+      /* Logged into another member of the login group */
       return 0;
     }
   }
-#endif
-  zCookieName = mprintf("fossil-cc-%.10s", db_get("project-code","x"));
-  zCookieValue = P(zCookieName);
-  if( zCookieValue && atoi(zCookieValue)==1 ) return 0;
-  if( captcha_is_correct(0) ){
-    cgi_set_cookie(zCookieName, "1", login_cookie_path(), 8*3600);
-    return 0;
-  }
 
   /* This appears to be a spider.  Offer the captcha */
   style_set_current_feature("captcha");
-  style_header("Verification");
-  cgi_printf("<form method='POST' action='%s'>\n",(g.zPath));
-  cgi_query_parameters_to_hidden();
-  cgi_printf("<p>Please demonstrate that you are human, not a spider or robot</p>\n");
-  captcha_generate(1);
+  style_header("I think you are a robot");
+  style_submenu_enable(0);
+  cgi_printf("<form method='POST' action='%R/ityaar'>\n"
+         "<p>You seem like a robot.\n"
+         "\n"
+         "<p>If you are human, you can prove that by solving the captcha below,\n"
+         "after which you will be allowed to proceed.\n");
+  if( bTest ){
+    cgi_printf("<input type=\"hidden\" name=\"istest\" value=\"1\">\n");
+  }
+  captcha_generate(3);
   cgi_printf("</form>\n");
-  style_finish_page();
+  if( !bTest ){
+    if( P("fossil-goto")==0 ){
+      cgi_set_cookie("fossil-goto", cgi_reconstruct_original_url(), 0, 600);
+    }
+    cgi_append_header("X-Robot: 1\r\n");
+    style_finish_page();
+  }
   return 1;
 }
 
+/*
+** WEBPAGE: ityaar
+**
+** This is the action for the form that is the captcha.  Not intended
+** for external use.  "ityaar" is an acronym "I Think You Are A Robot".
+**
+** If the captcha is correctly solved, then an anonymous login cookie
+** is set.  Regardless of whether or not the captcha was solved, this
+** page always redirects to the fossil-goto cookie.
+*/
+void captcha_callback(void){
+  int bTest = atoi(PD("istest","0"));
+  if( captcha_is_correct(1) ){
+    if( bTest==0 ){
+      if( !login_cookie_wellformed() ){
+        /* ^^^^--- Don't overwrite a valid login on another repo! */
+        login_set_anon_cookie(0, 0);
+      }
+      cgi_append_header("X-Robot: 0\r\n");
+    }
+    login_redirect_to_g();
+  }else{
+    g.isHuman = 0;
+    (void)exclude_spiders(bTest);
+    if( bTest ){
+      cgi_printf("<hr><p>Wrong code.  Try again\n");
+      style_finish_page();
+    }
+  }
+}
+
+
 /*
 ** Generate a WAV file that reads aloud the hex digits given by
 ** zHex.
@@ -689,7 +897,7 @@ static void captcha_wav(const char *zHex, Blob *pOut){
 */
 void captcha_wav_page(void){
   const char *zSeed = PD("name","0");
-  const char *zDecode = captcha_decode((unsigned int)atoi(zSeed));
+  const char *zDecode = captcha_decode((unsigned int)atoi(zSeed), 0);
   Blob audio;
   captcha_wav(zDecode, &audio);
   cgi_set_content_type("audio/wav");
