@@ -73,12 +73,14 @@
 # include <ws2tcpip.h>
 #else
 # include <sys/socket.h>
+# include <sys/un.h>
 # include <netinet/in.h>
 # include <arpa/inet.h>
 # include <sys/times.h>
 # include <sys/time.h>
 # include <sys/wait.h>
 # include <sys/select.h>
+# include <errno.h>
 #endif
 #ifdef __EMX__
   typedef int socklen_t;
@@ -901,6 +903,19 @@ void cgi_delete_query_parameter(const char *zName){
   }
 }
 
+/*
+** Return the number of query parameters.  Cookies and environment variables
+** do not count.  Also, do not count the special QP "name".
+*/
+int cgi_qp_count(void){
+  int cnt = 0;
+  int i;
+  for(i=0; i<nUsedQP; i++){
+    if( aParamQP[i].isQP && fossil_strcmp(aParamQP[i].zName,"name")!=0 ) cnt++;
+  }
+  return cnt;
+}
+
 /*
 ** Add an environment varaible value to the parameter set.  The zName
 ** portion is fixed but a copy is be made of zValue.
@@ -1885,6 +1900,30 @@ void cgi_query_parameters_to_url(HQuery *p){
   }
 }
 
+/*
+** Reconstruct the URL into memory obtained from fossil_malloc() and
+** return a pointer to that URL.
+*/
+char *cgi_reconstruct_original_url(void){
+  int i;
+  char cSep = '?';
+  Blob url;
+  blob_init(&url, 0, 0);
+  blob_appendf(&url, "%s/%s", g.zBaseURL, g.zPath);
+  for(i=0; i<nUsedQP; i++){
+    if( aParamQP[i].isQP ){
+      struct QParam *p = &aParamQP[i];
+      if( p->zValue && p->zValue[0] ){
+        blob_appendf(&url, "%c%t=%t", cSep, p->zName, p->zValue);
+      }else{
+        blob_appendf(&url, "%c%t", cSep, p->zName);
+      }
+      cSep = '&';
+    }
+  }
+  return blob_str(&url);
+}
+
 /*
 ** Tag query parameter zName so that it is not exported by
 ** cgi_query_parameters_to_hidden().  Or if zName==0, then
@@ -2086,7 +2125,7 @@ void cgi_handle_http_request(const char *zIpAddr){
   cgi_setenv("PATH_INFO", zToken);
   cgi_setenv("QUERY_STRING", &zToken[i]);
   if( zIpAddr==0 ){
-    zIpAddr = cgi_remote_ip(fileno(g.httpIn));
+    zIpAddr = cgi_remote_ip(fossil_fileno(g.httpIn));
   }
   if( zIpAddr ){
     cgi_setenv("REMOTE_ADDR", zIpAddr);
@@ -2441,6 +2480,7 @@ void cgi_handle_scgi_request(void){
 #define HTTP_SERVER_HAD_CHECKOUT   0x0008     /* Was a checkout open? */
 #define HTTP_SERVER_REPOLIST       0x0010     /* Allow repo listing */
 #define HTTP_SERVER_NOFORK         0x0020     /* Do not call fork() */
+#define HTTP_SERVER_UNIXSOCKET     0x0040     /* Use a unix-domain socket */
 
 #endif /* INTERFACE */
 
@@ -2481,33 +2521,81 @@ int cgi_http_server(
   int nchildren = 0;           /* Number of child processes */
   struct timeval delay;        /* How long to wait inside select() */
   struct sockaddr_in inaddr;   /* The socket address */
+  struct sockaddr_un uxaddr;   /* The address for unix-domain sockets */
   int opt = 1;                 /* setsockopt flag */
-  int iPort = mnPort;
+  int rc;                      /* Result code from system calls */
+  int iPort = mnPort;          /* Port to try to use */
 
   while( iPort<=mxPort ){
-    memset(&inaddr, 0, sizeof(inaddr));
-    inaddr.sin_family = AF_INET;
-    if( zIpAddr ){
-      inaddr.sin_addr.s_addr = inet_addr(zIpAddr);
-      if( inaddr.sin_addr.s_addr == INADDR_NONE ){
-        fossil_fatal("not a valid IP address: %s", zIpAddr);
+    if( flags & HTTP_SERVER_UNIXSOCKET ){
+      /* Initialize a Unix socket named g.zSockName */
+      assert( g.zSockName!=0 );
+      memset(&uxaddr, 0, sizeof(uxaddr));
+      if( strlen(g.zSockName)>sizeof(uxaddr.sun_path) ){
+        fossil_fatal("name of unix socket too big: %s\nmax size: %d\n",
+                     g.zSockName, (int)sizeof(uxaddr.sun_path));
+      }
+      if( file_isdir(g.zSockName, ExtFILE)!=0 ){
+        if( !file_issocket(g.zSockName) ){
+          fossil_fatal("cannot name socket \"%s\" because another object"
+                       " with that name already exists", g.zSockName);
+        }else{
+          unlink(g.zSockName);
+        }
+      }
+      uxaddr.sun_family = AF_UNIX;
+      strncpy(uxaddr.sun_path, g.zSockName, sizeof(uxaddr.sun_path)-1);
+      listener = socket(AF_UNIX, SOCK_STREAM, 0);
+      if( listener<0 ){
+        fossil_fatal("unable to create a unix socket named %s",
+                     g.zSockName);
+      }
+      /* Set the access permission for the new socket.  Default to 0660.
+      ** But use an alternative specified by --socket-mode if available.
+      ** Do this before bind() to avoid a race condition. */
+      if( g.zSockMode ){
+        file_set_mode(g.zSockName, listener, g.zSockMode, 0);
+      }else{
+        file_set_mode(g.zSockName, listener, "0660", 1);
       }
-    }else if( flags & HTTP_SERVER_LOCALHOST ){
-      inaddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
     }else{
-      inaddr.sin_addr.s_addr = htonl(INADDR_ANY);
-    }
-    inaddr.sin_port = htons(iPort);
-    listener = socket(AF_INET, SOCK_STREAM, 0);
-    if( listener<0 ){
-      iPort++;
-      continue;
+      /* Initialize a TCP/IP socket on port iPort */
+      memset(&inaddr, 0, sizeof(inaddr));
+      inaddr.sin_family = AF_INET;
+      if( zIpAddr ){
+        inaddr.sin_addr.s_addr = inet_addr(zIpAddr);
+        if( inaddr.sin_addr.s_addr == INADDR_NONE ){
+          fossil_fatal("not a valid IP address: %s", zIpAddr);
+        }
+      }else if( flags & HTTP_SERVER_LOCALHOST ){
+        inaddr.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
+      }else{
+        inaddr.sin_addr.s_addr = htonl(INADDR_ANY);
+      }
+      inaddr.sin_port = htons(iPort);
+      listener = socket(AF_INET, SOCK_STREAM, 0);
+      if( listener<0 ){
+        iPort++;
+        continue;
+      }
     }
 
     /* if we can't terminate nicely, at least allow the socket to be reused */
     setsockopt(listener,SOL_SOCKET,SO_REUSEADDR,&opt,sizeof(opt));
-
-    if( bind(listener, (struct sockaddr*)&inaddr, sizeof(inaddr))<0 ){
+  
+    if( flags & HTTP_SERVER_UNIXSOCKET ){
+      rc = bind(listener, (struct sockaddr*)&uxaddr, sizeof(uxaddr));
+      /* Set the owner of the socket if requested by --socket-owner.  This
+      ** must wait until after bind(), after the filesystem object has been
+      ** created.  See https://lkml.org/lkml/2004/11/1/84 and
+      ** https://fossil-scm.org/forum/forumpost/7517680ef9684c57 */
+      if( g.zSockOwner ){
+        file_set_owner(g.zSockName, listener, g.zSockOwner);
+      }
+    }else{
+      rc = bind(listener, (struct sockaddr*)&inaddr, sizeof(inaddr));
+    }
+    if( rc<0 ){
       close(listener);
       iPort++;
       continue;
@@ -2515,7 +2603,9 @@ int cgi_http_server(
     break;
   }
   if( iPort>mxPort ){
-    if( mnPort==mxPort ){
+    if( flags & HTTP_SERVER_UNIXSOCKET ){
+      fossil_fatal("unable to listen on unix socket %s", zIpAddr);
+    }else if( mnPort==mxPort ){
       fossil_fatal("unable to open listening socket on port %d", mnPort);
     }else{
       fossil_fatal("unable to open listening socket on any"
@@ -2524,11 +2614,17 @@ int cgi_http_server(
   }
   if( iPort>mxPort ) return 1;
   listen(listener,10);
-  fossil_print("Listening for %s requests on TCP port %d\n",
-     (flags & HTTP_SERVER_SCGI)!=0 ? "SCGI" :
-        g.httpUseSSL?"TLS-encrypted HTTPS":"HTTP",  iPort);
+  if( flags & HTTP_SERVER_UNIXSOCKET ){
+    fossil_print("Listening for %s requests on unix socket %s\n",
+       (flags & HTTP_SERVER_SCGI)!=0 ? "SCGI" :
+          g.httpUseSSL?"TLS-encrypted HTTPS":"HTTP",  g.zSockName);
+  }else{
+    fossil_print("Listening for %s requests on TCP port %d\n",
+       (flags & HTTP_SERVER_SCGI)!=0 ? "SCGI" :
+          g.httpUseSSL?"TLS-encrypted HTTPS":"HTTP",  iPort);
+  }
   fflush(stdout);
-  if( zBrowser ){
+  if( zBrowser && (flags & HTTP_SERVER_UNIXSOCKET)==0 ){
     assert( strstr(zBrowser,"%d")!=0 );
     zBrowser = mprintf(zBrowser /*works-like:"%d"*/, iPort);
 #if defined(__CYGWIN__)
@@ -2574,6 +2670,7 @@ int cgi_http_server(
           close(connection);
         }else{
           int nErr = 0, fd;
+          g.zSockName = 0 /* avoid deleting the socket via atexit() */;
           close(0);
           fd = dup(connection);
           if( fd!=0 ) nErr++;
