@@ -150,7 +150,7 @@ static void chat_emit_alert_list(void){
 **
 ** This is the main page that humans use to access the chatroom.  Simply
 ** point a web-browser at /chat and the screen fills with the latest
-** chat messages, and waits for new one.
+** chat messages, and waits for new ones.
 **
 ** Other /chat-OP pages are used by XHR requests from this page to
 ** send new chat message, delete older messages, or poll for changes.
@@ -159,23 +159,6 @@ void chat_webpage(void){
   char *zAlert;
   char *zProjectName;
   char * zInputPlaceholder0;  /* Common text input placeholder value */
-  const char *zPaperclip =
-    "<svg height=\"8.0\" width=\"16.0\"><path "
-    "stroke=\"rgb(100,100,100)\" "
-    "d=\"M 15.93452,3.2530441 "
-    "A 4.1499493,4.1265346 0 0 0 11.804809,6.5256284e-4 H 2.8582923 A "
-    "2.8239899,2.8080565 0 0 0 0.68965668,0.96142476 2.874599,2.8583801 "
-    "0 0 0 0.03119302,3.2388108 2.7632589,2.7476682 0 0 0 "
-    "0.81132923,4.7689293 3.168132,3.1502569 0 0 0 3.0300653,5.66565 l "
-    "7.7297897,-4e-7 a 1.6802234,1.6707433 0 0 0 0.0072,-3.3377933 H "
-    "5.6138192 v 1.0105899 l 5.1460358,-0.00712 a 0.66804062,0.66427143 "
-    "0 0 1 0,1.3237305 l -7.7226325,0.00712 A 2.0243655,2.0129437 0 0 1 "
-    "1.0332029,3.0964741 1.8522944,1.8418435 0 0 1 2.8511351,1.0041257 h "
-    "8.9465169 a 3.1478884,3.1301275 0 0 1 3.134859,2.4339559 3.0365483,"
-    "3.0194156 0 0 1 -0.629835,2.4908908 3.0365483,3.0194156 0 0 1 "
-    "-2.31178,1.0746415 l -7.5437026,-0.014233 -0.00716,1.0034736 "
-    "7.5365456,0.00715 a 4.048731,4.0258875 0 0 0 3.957938,-4.7469259 z\""
-    "/></svg>";
 
   login_check_credentials();
   if( !g.perm.Chat ){
@@ -206,8 +189,10 @@ void chat_webpage(void){
          "    <div id='chat-buttons-wrapper'>\n"
          "      <span class='cbutton' id=\"chat-button-preview\" "
          "        title=\"Preview message (Shift-Enter)\">&#128065;</span>\n"
+         "      <span class='cbutton' id=\"chat-button-search\" "
+         "        title=\"Search chat history\">&#x1f50d;</span>\n"
          "      <span class='cbutton' id=\"chat-button-attach\" "
-         "        title=\"Attach file to message\">%s</span>\n"
+         "        title=\"Attach file to message\">&#x1f4ce;</span>\n"
          "      <span class='cbutton' id=\"chat-button-settings\" "
          "        title=\"Configure chat\">&#9881;</span>\n"
          "      <span class='cbutton' id=\"chat-button-submit\" "
@@ -236,13 +221,21 @@ void chat_webpage(void){
          "</div>\n"
          "<div id='chat-preview' class='hidden chat-view'>\n"
          " <header>Preview: (<a href='%R/md_rules' target='_blank'>markdown reference</a>)</header>\n"
-         " <div id='chat-preview-content' class='message-widget-content'></div>\n"
-         " <div id='chat-preview-buttons'><button id='chat-preview-close'>Close Preview</button></div>\n"
+         " <div id='chat-preview-content'></div>\n"
+         " <div class='button-bar'><button class='action-close'>Close Preview</button></div>\n"
          "</div>\n"
          "<div id='chat-config' class='hidden chat-view'>\n"
-         "<div id='chat-config-options'></div>\n",(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0),(zPaperclip));
+         "<div id='chat-config-options'></div>\n",(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0),(zInputPlaceholder0));
     /* ^^^populated client-side */
-  cgi_printf("<button>Close Settings</button>\n"
+  cgi_printf("<div class='button-bar'><button class='action-close'>Close Settings</button></div>\n"
+         "</div>\n"
+         "<div id='chat-search' class='hidden chat-view'>\n"
+         "  <div id='chat-search-content'></div>\n");
+      /* ^^^populated client-side */
+  cgi_printf("  <div class='button-bar'>\n"
+         "    <button class='action-clear'>Clear results</button>\n"
+         "    <button class='action-close'>Close Search</button>\n"
+         "  </div>\n"
          "</div>\n"
          "<div id='chat-messages-wrapper' class='chat-view'>\n");
   /* New chat messages get inserted immediately after this element */
@@ -274,7 +267,8 @@ void chat_webpage(void){
   style_finish_page();
 }
 
-/* Definition of repository tables used by chat
+/*
+** Definition of repository tables used by chat
 */
 static const char zChatSchema1[] =
 "CREATE TABLE repository.chat(\n"
@@ -292,8 +286,38 @@ static const char zChatSchema1[] =
 
 
 /*
-** Make sure the repository data tables used by chat exist.  Create them
-** if they do not.
+** Create or rebuild the /chat search index. Requires that the
+** repository.chat table exists. If bForce is true, it will drop the
+** chatfts1 table and recreate/reindex it. If bForce is 0, it will
+** only index the chat content if the chatfts1 table does not already
+** exist.
+*/
+void chat_rebuild_index(int bForce){
+  if( bForce!=0 ){
+    db_multi_exec("DROP TABLE IF EXISTS chatfts1");
+  }
+  if( bForce!=0 || !db_table_exists("repository", "chatfts1") ){
+    const int tokType = search_tokenizer_type(0);
+    const char *zTokenizer = search_tokenize_arg_for_type(
+      tokType==FTS5TOK_NONE ? FTS5TOK_PORTER : tokType
+      /* Special case: if fts search is disabled for the main repo
+      ** content, use a default tokenizer here. */
+    );
+    assert( zTokenizer && zTokenizer[0] );
+    db_multi_exec(
+      "CREATE VIRTUAL TABLE repository.chatfts1 USING fts5("
+      "    xmsg, content=chat, content_rowid=msgid%s"
+      ");"
+      "INSERT INTO repository.chatfts1(chatfts1) VALUES('rebuild');",
+      zTokenizer/*safe-for-%s*/
+    );
+  }
+}
+
+/*
+** Make sure the repository data tables used by chat exist.  Create
+** them if they do not. Set up TEMP triggers (if needed) to update the
+** chatfts1 table as the chat table is updated.
 */
 static void chat_create_tables(void){
   if( !db_table_exists("repository","chat") ){
@@ -304,6 +328,16 @@ static void chat_create_tables(void){
     }
     db_multi_exec("ALTER TABLE chat ADD COLUMN lmtime TEXT");
   }
+  chat_rebuild_index(0);
+  db_multi_exec(
+    "CREATE TEMP TRIGGER IF NOT EXISTS chat_ai AFTER INSERT ON chat BEGIN "
+    "  INSERT INTO chatfts1(rowid, xmsg) VALUES(new.msgid, new.xmsg);"
+    "END;"
+    "CREATE TEMP TRIGGER IF NOT EXISTS chat_ad AFTER DELETE ON chat BEGIN "
+    "  INSERT INTO chatfts1(chatfts1, rowid, xmsg) "
+    "    VALUES('delete', old.msgid, old.xmsg);"
+    "END;"
+  );
 }
 
 /*
@@ -391,12 +425,12 @@ void chat_send_webpage(void){
     chat_emit_permissions_error(0);
     return;
   }
-  chat_create_tables();
   zUserName = (g.zLogin && g.zLogin[0]) ? g.zLogin : "nobody";
   nByte = atoi(PD("file:bytes","0"));
   zMsg = PD("msg","");
   db_begin_write();
   db_unprotect(PROTECT_READONLY);
+  chat_create_tables();
   chat_purge();
   if( nByte==0 ){
     if( zMsg[0] ){
@@ -420,8 +454,8 @@ void chat_send_webpage(void){
     db_finalize(&q);
     blob_reset(&b);
   }
-  db_protect_pop();
   db_commit_transaction();
+  db_protect_pop();
 }
 
 /*
@@ -455,25 +489,117 @@ static char *chat_format_to_html(const char *zMsg, int isWiki){
 /*
 ** COMMAND: test-chat-formatter
 **
-** Usage: %fossil test-chat-formatter STRING ...
+** Usage: %fossil test-chat-formatter ?OPTIONS? STRING ...
 **
 ** Transform each argument string into HTML that will display the
 ** chat message.  This is used to test the formatter and to verify
 ** that a malicious message text will not cause HTML or JS injection
 ** into the chat display in a browser.
+**
+** Options:
+**
+**     -w|--wiki     Assume fossil wiki format instead of markdown
 */
 void chat_test_formatter_cmd(void){
   int i;
   char *zOut;
+  int const isWiki = find_option("w","wiki",0)!=0;
   db_find_and_open_repository(0,0);
   g.perm.Hyperlink = 1;
-  for(i=0; i<g.argc; i++){
-    zOut = chat_format_to_html(g.argv[i], 0);
-    fossil_print("[%d]: %s\n", i, zOut);
+  for(i=2; i<g.argc; i++){
+    zOut = chat_format_to_html(g.argv[i], isWiki);
+    fossil_print("[%d]: %s\n", i-1, zOut);
     fossil_free(zOut);
   }
 }
 
+/*
+** The SQL statement passed as the first argument should return zero or
+** more rows of data, each of which represents a single message from the
+** "chat" table. The rows returned should be similar to those returned
+** by:
+**
+**     SELECT msgid, 
+**            datetime(mtime), 
+**            xfrom, 
+**            xmsg, 
+**            octet_length(file),"
+**            fname, 
+**            fmime, 
+**            mdel, 
+**            lmtime
+**     FROM chat;
+**
+** This function loops through all rows returned by statement p, adding
+** a record to the JSON stored in argument pJson for each. See comments 
+** above function chat_poll_webpage() for a description of the JSON records
+** added to pJson.
+*/
+static int chat_poll_rowstojson(
+  Stmt *p,                        /* Statement to read rows from */
+  int bRaw,                       /* True to return raw format xmsg */
+  Blob *pJson                     /* Append json array entries here */
+){
+  int cnt = 0;
+  const char *zChatUser = db_get("chat-timeline-user",0);
+  while( db_step(p)==SQLITE_ROW ){
+    int isWiki = 0;             /* True if chat message is x-fossil-wiki */
+    int id = db_column_int(p, 0);
+    const char *zDate = db_column_text(p, 1);
+    const char *zFrom = db_column_text(p, 2);
+    const char *zRawMsg = db_column_text(p, 3);
+    int nByte = db_column_int(p, 4);
+    const char *zFName = db_column_text(p, 5);
+    const char *zFMime = db_column_text(p, 6);
+    int iToDel = db_column_int(p, 7);
+    const char *zLMtime = db_column_text(p, 8);
+    char *zMsg;
+    if(cnt++){
+      blob_append(pJson, ",\n", 2);
+    }
+    blob_appendf(pJson, "{\"msgid\":%d,", id);
+    blob_appendf(pJson, "\"mtime\":\"%.10sT%sZ\",", zDate, zDate+11);
+    if( zLMtime && zLMtime[0] ){
+      blob_appendf(pJson, "\"lmtime\":%!j,", zLMtime);
+    }
+    blob_append(pJson, "\"xfrom\":", -1);
+    if(zFrom){
+      blob_appendf(pJson, "%!j,", zFrom);
+      isWiki = fossil_strcmp(zFrom,zChatUser)==0;
+    }else{
+      /* see https://fossil-scm.org/forum/forumpost/e0be0eeb4c */
+      blob_appendf(pJson, "null,");
+      isWiki = 0;
+    }
+    blob_appendf(pJson, "\"uclr\":%!j,",
+        isWiki ? "transparent" : user_color(zFrom ? zFrom : "nobody"));
+
+    if(bRaw){
+      blob_appendf(pJson, "\"xmsg\":%!j,", zRawMsg);
+    }else{
+      zMsg = chat_format_to_html(zRawMsg ? zRawMsg : "", isWiki);
+      blob_appendf(pJson, "\"xmsg\":%!j,", zMsg);
+      fossil_free(zMsg);
+    }
+
+    if( nByte==0 ){
+      blob_appendf(pJson, "\"fsize\":0");
+    }else{
+      blob_appendf(pJson, "\"fsize\":%d,\"fname\":%!j,\"fmime\":%!j",
+          nByte, zFName, zFMime);
+    }
+
+    if( iToDel ){
+      blob_appendf(pJson, ",\"mdel\":%d}", iToDel);
+    }else{
+      blob_append(pJson, "}", 1);
+    }
+  }
+  db_reset(p);
+
+  return cnt;
+}
+
 /*
 ** WEBPAGE: chat-poll hidden loadavg-exempt
 **
@@ -571,8 +697,6 @@ void chat_poll_webpage(void){
   sqlite3_int64 dataVersion;  /* Data version.  Used for polling. */
   const int iDelay = 1000;    /* Delay until next poll (milliseconds) */
   int nDelay;                 /* Maximum delay.*/
-  const char *zChatUser;      /* chat-timeline-user */
-  int isWiki = 0;             /* True if chat message is x-fossil-wiki */
   int msgid = atoi(PD("name","0"));
   const int msgBefore = atoi(PD("before","0"));
   int nLimit = msgBefore>0 ? atoi(PD("n","0")) : 0;
@@ -586,7 +710,6 @@ void chat_poll_webpage(void){
     chat_emit_permissions_error(1);
     return;
   }
-  zChatUser = db_get("chat-timeline-user",0);
   chat_create_tables();
   cgi_set_content_type("application/json");
   dataVersion = db_int64(0, "PRAGMA data_version");
@@ -626,60 +749,7 @@ void chat_poll_webpage(void){
   blob_reset(&sql);
   blob_init(&json, "{\"msgs\":[\n", -1);
   while( nDelay>0 ){
-    int cnt = 0;
-    while( db_step(&q1)==SQLITE_ROW ){
-      int id = db_column_int(&q1, 0);
-      const char *zDate = db_column_text(&q1, 1);
-      const char *zFrom = db_column_text(&q1, 2);
-      const char *zRawMsg = db_column_text(&q1, 3);
-      int nByte = db_column_int(&q1, 4);
-      const char *zFName = db_column_text(&q1, 5);
-      const char *zFMime = db_column_text(&q1, 6);
-      int iToDel = db_column_int(&q1, 7);
-      const char *zLMtime = db_column_text(&q1, 8);
-      char *zMsg;
-      if(cnt++){
-        blob_append(&json, ",\n", 2);
-      }
-      blob_appendf(&json, "{\"msgid\":%d,", id);
-      blob_appendf(&json, "\"mtime\":\"%.10sT%sZ\",", zDate, zDate+11);
-      if( zLMtime && zLMtime[0] ){
-        blob_appendf(&json, "\"lmtime\":%!j,", zLMtime);
-      }
-      blob_append(&json, "\"xfrom\":", -1);
-      if(zFrom){
-        blob_appendf(&json, "%!j,", zFrom);
-        isWiki = fossil_strcmp(zFrom,zChatUser)==0;
-      }else{
-        /* see https://fossil-scm.org/forum/forumpost/e0be0eeb4c */
-        blob_appendf(&json, "null,");
-        isWiki = 0;
-      }
-      blob_appendf(&json, "\"uclr\":%!j,",
-                 isWiki ? "transparent" : user_color(zFrom ? zFrom : "nobody"));
-
-      if(bRaw){
-        blob_appendf(&json, "\"xmsg\":%!j,", zRawMsg);
-      }else{
-        zMsg = chat_format_to_html(zRawMsg ? zRawMsg : "", isWiki);
-        blob_appendf(&json, "\"xmsg\":%!j,", zMsg);
-        fossil_free(zMsg);
-      }
-
-      if( nByte==0 ){
-        blob_appendf(&json, "\"fsize\":0");
-      }else{
-        blob_appendf(&json, "\"fsize\":%d,\"fname\":%!j,\"fmime\":%!j",
-               nByte, zFName, zFMime);
-      }
-
-      if( iToDel ){
-        blob_appendf(&json, ",\"mdel\":%d}", iToDel);
-      }else{
-        blob_append(&json, "}", 1);
-      }
-    }
-    db_reset(&q1);
+    int cnt = chat_poll_rowstojson(&q1, bRaw, &json);
     if( cnt || msgBefore>0 ){
       break;
     }
@@ -699,6 +769,73 @@ void chat_poll_webpage(void){
   return;
 }
 
+
+/*
+** WEBPAGE: chat-query hidden loadavg-exempt
+*/
+void chat_query_webpage(void){
+  Blob json;                  /* The json to be constructed and returned */
+  Blob sql = empty_blob;
+  Stmt q1;
+  int nLimit = atoi(PD("n","500"));
+  int iFirst = atoi(PD("i","0"));
+  const char *zQuery = PD("q", "");
+  i64 iMin = 0;
+  i64 iMax = 0;
+
+  login_check_credentials();
+  if( !g.perm.Chat ) {
+    chat_emit_permissions_error(1);
+    return;
+  }
+  chat_create_tables();
+  cgi_set_content_type("application/json");
+
+  if( zQuery[0] ){
+    iMax = db_int64(0, "SELECT max(msgid) FROM chat");
+    iMin = db_int64(0, "SELECT min(msgid) FROM chat");
+    if( '#'==zQuery[0] ){
+      /* Assume we're looking for an exact msgid match. */
+      ++zQuery;
+      blob_append_sql(&sql,
+        "SELECT msgid, datetime(mtime), xfrom, "
+        "  xmsg, octet_length(file), fname, fmime, mdel, lmtime "
+        "  FROM chat WHERE msgid=+%Q",
+        zQuery
+      );
+    }else{
+      char * zPat = search_simplify_pattern(zQuery);
+      blob_append_sql(&sql,
+        "SELECT * FROM ("
+        "SELECT c.msgid, datetime(c.mtime), c.xfrom, "
+        "  highlight(chatfts1, 0, '<span class=\"match\">', '</span>'), "
+        "  octet_length(c.file), c.fname, c.fmime, c.mdel, c.lmtime "
+        "  FROM chatfts1(%Q) f, chat c "
+        "  WHERE f.rowid=c.msgid"
+        "  ORDER BY f.rowid DESC LIMIT %d"
+        ") ORDER BY 1 ASC", zPat, nLimit
+      );
+      fossil_free(zPat);
+    }
+  }else{
+    blob_append_sql(&sql,
+        "SELECT msgid, datetime(mtime), xfrom, "
+        "  xmsg, octet_length(file), fname, fmime, mdel, lmtime"
+        "  FROM chat WHERE msgid>=%d LIMIT %d",
+        iFirst, nLimit
+    );
+  }
+
+  db_prepare(&q1, "%s", blob_sql_text(&sql));
+  blob_reset(&sql);
+  blob_init(&json, "{\"msgs\":[\n", -1);
+  chat_poll_rowstojson(&q1, 0, &json);
+  db_finalize(&q1);
+  blob_appendf(&json, "\n], \"first\":%lld, \"last\":%lld}", iMin, iMax);
+  cgi_set_content(&json);
+  return;
+}
+
 /*
 ** WEBPAGE: chat-fetch-one hidden loadavg-exempt
 **
@@ -808,8 +945,10 @@ void chat_fetch_one(void){
 */
 void chat_download_webpage(void){
   int msgid;
-  Blob r;
-  const char *zMime;
+  int bCheckedMimetype = 0;  /* true to bypass the text/... mimetype
+                             ** check at the end */
+  Blob r;                    /* file content */
+  const char *zMime;         /* file mimetype */
   const char *zName = PD("name","0");
   login_check_credentials();
   if( !g.perm.Chat ){
@@ -836,11 +975,13 @@ void chat_download_webpage(void){
       markdown_to_html(&r, 0, &r2);
       safe_html(&r2);
       zMime2 = "text/html";
+      bCheckedMimetype = 1;
     }else if(fossil_strcmp(zMime, "text/x-fossil-wiki")==0
              || sqlite3_strglob("*.wiki", zName)==0){
       /* .wiki files get uploaded as application/octet-stream */
       wiki_convert(&r, &r2, 0);
       zMime2 = "text/html";
+      bCheckedMimetype = 1;
     }else if(fossil_strcmp(zMime, "text/x-pikchr")==0
              || sqlite3_strglob("*.pikchr",zName)==0){
       /* .pikchr files get uploaded as application/octet-stream */
@@ -852,6 +993,7 @@ void chat_download_webpage(void){
       }
       zMime2 = w>0 ? "image/svg+xml" : "text/html";
       free(zOut);
+      bCheckedMimetype = 1;
     }
     if(r2.aData!=0){
       blob_swap(&r, &r2);
@@ -859,6 +1001,18 @@ void chat_download_webpage(void){
       zMime = zMime2;
     }
   }
+  if( bCheckedMimetype==0 && sqlite3_strglob("text/*", zMime)==0 ){
+    /* The problem: both Chrome and Firefox upload *.patch with
+    ** the mimetype text/x-patch, whereas we very often use that
+    ** name glob for fossil-format patches. That causes such files
+    ** to attempt to render in the browser when clicked via
+    ** download links in chat.
+    **
+    ** The workaround: */
+    if( looks_like_binary(&r) ){
+      zMime = "application/octet-stream";
+    }
+  }
   cgi_set_content_type(zMime);
   cgi_set_content(&r);
 }
@@ -986,8 +1140,7 @@ void chat_msg_from_event(
     fossil_free(zUuid);
   }else if( zType[0]=='w' ){
     /* Wiki page changes */
-    char *zUuid;
-    zUuid = db_text(0, "SELECT uuid FROM blob WHERE rid=%d", rid);
+    char *zUuid = rid_to_uuid(rid);
     wiki_hyperlink_override(zUuid);
     if( zMsg[0]=='-' ){
       zRes = mprintf("Delete wiki page <a href='%R/whistory?name=%t'>%h</a>",
@@ -1004,6 +1157,13 @@ void chat_msg_from_event(
     }
     wiki_hyperlink_override(0);
     fossil_free(zUuid);
+  }else if( zType[0]=='f' ){
+    /* Forum changes */
+    char *zUuid = rid_to_uuid(rid);
+    zRes = mprintf( "%W (artifact: <a href='%R/info/%S'>%S</a>, "
+                    "user: <a href='%R/timeline?u=%t&c=%S'>%h</a>)",
+                    zMsg, zUuid, zUuid, zUser, zUuid, zUser);
+    fossil_free(zUuid);
   }else{
     /* Anything else */
     zRes = mprintf("%W", zMsg);
@@ -1112,7 +1272,7 @@ void chat_command(void){
 #endif
     fossil_system(zCmd);
   }else if( strcmp(g.argv[2],"send")==0 ){
-    const char *zFilename = find_option("file","r",1);
+    const char *zFilename = find_option("file","f",1);
     const char *zAs = find_option("as",0,1);
     const char *zMsg = find_option("message","m",1);
     int allowUnsafe = find_option("unsafe",0,0)!=0;
