@@ -34,6 +34,113 @@
 #define COMMENT_PRINT_UNSET      (-1)              /* Not initialized. */
 #endif
 
+/********* Code copied from SQLite src/shell.c.in on 2024-09-30 **********/
+/* Lookup table to estimate the number of columns consumed by a Unicode
+** character.
+*/
+static const struct {
+  unsigned char w;    /* Width of the character in columns */
+  int iFirst;         /* First character in a span having this width */
+} aUWidth[] = {
+   /* {1, 0x00000}, */
+  {0, 0x00300},  {1, 0x00370},  {0, 0x00483},  {1, 0x00487},  {0, 0x00488},
+  {1, 0x0048a},  {0, 0x00591},  {1, 0x005be},  {0, 0x005bf},  {1, 0x005c0},
+  {0, 0x005c1},  {1, 0x005c3},  {0, 0x005c4},  {1, 0x005c6},  {0, 0x005c7},
+  {1, 0x005c8},  {0, 0x00600},  {1, 0x00604},  {0, 0x00610},  {1, 0x00616},
+  {0, 0x0064b},  {1, 0x0065f},  {0, 0x00670},  {1, 0x00671},  {0, 0x006d6},
+  {1, 0x006e5},  {0, 0x006e7},  {1, 0x006e9},  {0, 0x006ea},  {1, 0x006ee},
+  {0, 0x0070f},  {1, 0x00710},  {0, 0x00711},  {1, 0x00712},  {0, 0x00730},
+  {1, 0x0074b},  {0, 0x007a6},  {1, 0x007b1},  {0, 0x007eb},  {1, 0x007f4},
+  {0, 0x00901},  {1, 0x00903},  {0, 0x0093c},  {1, 0x0093d},  {0, 0x00941},
+  {1, 0x00949},  {0, 0x0094d},  {1, 0x0094e},  {0, 0x00951},  {1, 0x00955},
+  {0, 0x00962},  {1, 0x00964},  {0, 0x00981},  {1, 0x00982},  {0, 0x009bc},
+  {1, 0x009bd},  {0, 0x009c1},  {1, 0x009c5},  {0, 0x009cd},  {1, 0x009ce},
+  {0, 0x009e2},  {1, 0x009e4},  {0, 0x00a01},  {1, 0x00a03},  {0, 0x00a3c},
+  {1, 0x00a3d},  {0, 0x00a41},  {1, 0x00a43},  {0, 0x00a47},  {1, 0x00a49},
+  {0, 0x00a4b},  {1, 0x00a4e},  {0, 0x00a70},  {1, 0x00a72},  {0, 0x00a81},
+  {1, 0x00a83},  {0, 0x00abc},  {1, 0x00abd},  {0, 0x00ac1},  {1, 0x00ac6},
+  {0, 0x00ac7},  {1, 0x00ac9},  {0, 0x00acd},  {1, 0x00ace},  {0, 0x00ae2},
+  {1, 0x00ae4},  {0, 0x00b01},  {1, 0x00b02},  {0, 0x00b3c},  {1, 0x00b3d},
+  {0, 0x00b3f},  {1, 0x00b40},  {0, 0x00b41},  {1, 0x00b44},  {0, 0x00b4d},
+  {1, 0x00b4e},  {0, 0x00b56},  {1, 0x00b57},  {0, 0x00b82},  {1, 0x00b83},
+  {0, 0x00bc0},  {1, 0x00bc1},  {0, 0x00bcd},  {1, 0x00bce},  {0, 0x00c3e},
+  {1, 0x00c41},  {0, 0x00c46},  {1, 0x00c49},  {0, 0x00c4a},  {1, 0x00c4e},
+  {0, 0x00c55},  {1, 0x00c57},  {0, 0x00cbc},  {1, 0x00cbd},  {0, 0x00cbf},
+  {1, 0x00cc0},  {0, 0x00cc6},  {1, 0x00cc7},  {0, 0x00ccc},  {1, 0x00cce},
+  {0, 0x00ce2},  {1, 0x00ce4},  {0, 0x00d41},  {1, 0x00d44},  {0, 0x00d4d},
+  {1, 0x00d4e},  {0, 0x00dca},  {1, 0x00dcb},  {0, 0x00dd2},  {1, 0x00dd5},
+  {0, 0x00dd6},  {1, 0x00dd7},  {0, 0x00e31},  {1, 0x00e32},  {0, 0x00e34},
+  {1, 0x00e3b},  {0, 0x00e47},  {1, 0x00e4f},  {0, 0x00eb1},  {1, 0x00eb2},
+  {0, 0x00eb4},  {1, 0x00eba},  {0, 0x00ebb},  {1, 0x00ebd},  {0, 0x00ec8},
+  {1, 0x00ece},  {0, 0x00f18},  {1, 0x00f1a},  {0, 0x00f35},  {1, 0x00f36},
+  {0, 0x00f37},  {1, 0x00f38},  {0, 0x00f39},  {1, 0x00f3a},  {0, 0x00f71},
+  {1, 0x00f7f},  {0, 0x00f80},  {1, 0x00f85},  {0, 0x00f86},  {1, 0x00f88},
+  {0, 0x00f90},  {1, 0x00f98},  {0, 0x00f99},  {1, 0x00fbd},  {0, 0x00fc6},
+  {1, 0x00fc7},  {0, 0x0102d},  {1, 0x01031},  {0, 0x01032},  {1, 0x01033},
+  {0, 0x01036},  {1, 0x01038},  {0, 0x01039},  {1, 0x0103a},  {0, 0x01058},
+  {1, 0x0105a},  {2, 0x01100},  {0, 0x01160},  {1, 0x01200},  {0, 0x0135f},
+  {1, 0x01360},  {0, 0x01712},  {1, 0x01715},  {0, 0x01732},  {1, 0x01735},
+  {0, 0x01752},  {1, 0x01754},  {0, 0x01772},  {1, 0x01774},  {0, 0x017b4},
+  {1, 0x017b6},  {0, 0x017b7},  {1, 0x017be},  {0, 0x017c6},  {1, 0x017c7},
+  {0, 0x017c9},  {1, 0x017d4},  {0, 0x017dd},  {1, 0x017de},  {0, 0x0180b},
+  {1, 0x0180e},  {0, 0x018a9},  {1, 0x018aa},  {0, 0x01920},  {1, 0x01923},
+  {0, 0x01927},  {1, 0x01929},  {0, 0x01932},  {1, 0x01933},  {0, 0x01939},
+  {1, 0x0193c},  {0, 0x01a17},  {1, 0x01a19},  {0, 0x01b00},  {1, 0x01b04},
+  {0, 0x01b34},  {1, 0x01b35},  {0, 0x01b36},  {1, 0x01b3b},  {0, 0x01b3c},
+  {1, 0x01b3d},  {0, 0x01b42},  {1, 0x01b43},  {0, 0x01b6b},  {1, 0x01b74},
+  {0, 0x01dc0},  {1, 0x01dcb},  {0, 0x01dfe},  {1, 0x01e00},  {0, 0x0200b},
+  {1, 0x02010},  {0, 0x0202a},  {1, 0x0202f},  {0, 0x02060},  {1, 0x02064},
+  {0, 0x0206a},  {1, 0x02070},  {0, 0x020d0},  {1, 0x020f0},  {2, 0x02329},
+  {1, 0x0232b},  {2, 0x02e80},  {0, 0x0302a},  {2, 0x03030},  {1, 0x0303f},
+  {2, 0x03040},  {0, 0x03099},  {2, 0x0309b},  {1, 0x0a4d0},  {0, 0x0a806},
+  {1, 0x0a807},  {0, 0x0a80b},  {1, 0x0a80c},  {0, 0x0a825},  {1, 0x0a827},
+  {2, 0x0ac00},  {1, 0x0d7a4},  {2, 0x0f900},  {1, 0x0fb00},  {0, 0x0fb1e},
+  {1, 0x0fb1f},  {0, 0x0fe00},  {2, 0x0fe10},  {1, 0x0fe1a},  {0, 0x0fe20},
+  {1, 0x0fe24},  {2, 0x0fe30},  {1, 0x0fe70},  {0, 0x0feff},  {2, 0x0ff00},
+  {1, 0x0ff61},  {2, 0x0ffe0},  {1, 0x0ffe7},  {0, 0x0fff9},  {1, 0x0fffc},
+  {0, 0x10a01},  {1, 0x10a04},  {0, 0x10a05},  {1, 0x10a07},  {0, 0x10a0c},
+  {1, 0x10a10},  {0, 0x10a38},  {1, 0x10a3b},  {0, 0x10a3f},  {1, 0x10a40},
+  {0, 0x1d167},  {1, 0x1d16a},  {0, 0x1d173},  {1, 0x1d183},  {0, 0x1d185},
+  {1, 0x1d18c},  {0, 0x1d1aa},  {1, 0x1d1ae},  {0, 0x1d242},  {1, 0x1d245},
+  {2, 0x20000},  {1, 0x2fffe},  {2, 0x30000},  {1, 0x3fffe},  {0, 0xe0001},
+  {1, 0xe0002},  {0, 0xe0020},  {1, 0xe0080},  {0, 0xe0100},  {1, 0xe01f0}
+};
+
+/*
+** Return an estimate of the width, in columns, for the single Unicode
+** character c.  For normal characters, the answer is always 1.  But the
+** estimate might be 0 or 2 for zero-width and double-width characters.
+**
+** Different display devices display unicode using different widths.  So
+** it is impossible to know that true display width with 100% accuracy.
+** Inaccuracies in the width estimates might cause columns to be misaligned.
+** Unfortunately, there is nothing we can do about that.
+*/
+static int cli_wcwidth(int c){
+  int iFirst, iLast;
+
+  /* Fast path for common characters */
+  if( c<=0x300 ) return 1;
+
+  /* The general case */
+  iFirst = 0;
+  iLast = sizeof(aUWidth)/sizeof(aUWidth[0]) - 1;
+  while( iFirst<iLast-1 ){
+    int iMid = (iFirst+iLast)/2;
+    int cMid = aUWidth[iMid].iFirst;
+    if( cMid < c ){
+      iFirst = iMid;
+    }else if( cMid > c ){
+      iLast = iMid - 1;
+    }else{
+      return aUWidth[iMid].w;
+    }
+  }
+  if( aUWidth[iLast].iFirst > c ) return aUWidth[iFirst].w;
+  return aUWidth[iLast].w;
+}
+/******* End of code copied from SQLite *************************************/
+
 /*
 ** This is the previous value used by most external callers when they
 ** needed to specify a default maximum line length to be used with the
@@ -111,58 +218,95 @@ static int comment_check_orig(
 static int comment_next_space(
   const char *zLine, /* [in] The comment line being printed. */
   int index,         /* [in] The current character index being handled. */
-  int *distUTF8      /* [out] Distance to next space in UTF-8 sequences. */
+  int maxChars,      /* [in] Optimization hint to abort before space found. */
+  int *sumWidth      /* [out] Summated width of all characters to next space. */
 ){
-  int nextIndex = index + 1;
-  int fNonASCII=0;
+  int cchUTF8, utf32, wcwidth = 0;
+  int nextIndex = index;
   for(;;){
-    char c = zLine[nextIndex];
-    if( (c&0x80)==0x80 ) fNonASCII=1;
-    if( c==0 || fossil_isspace(c) ){
-      if( distUTF8 ){
-        if( fNonASCII!=0 ){
-          *distUTF8 = strlen_utf8(&zLine[index], nextIndex-index);
-        }else{
-          *distUTF8 = nextIndex-index;
-        }
-      }
+    char_info_utf8(&zLine[nextIndex],&cchUTF8,&utf32);
+    nextIndex += cchUTF8;
+    wcwidth += cli_wcwidth(utf32);
+    if( zLine[nextIndex]==0 || fossil_isspace(zLine[nextIndex]) ||
+        wcwidth>maxChars ){
+      *sumWidth = wcwidth;
       return nextIndex;
     }
-    nextIndex++;
   }
   return 0; /* NOT REACHED */
 }
 
 /*
-** Count the number of UTF-8 sequences in a string. Incomplete, ill-formed and
-** overlong sequences are counted as one sequence. The invalid lead bytes 0xC0
-** to 0xC1 and 0xF5 to 0xF7 are allowed to initiate (ill-formed) 2- and 4-byte
-** sequences, respectively, the other invalid lead bytes 0xF8 to 0xFF are
-** treated as invalid 1-byte sequences (as lone trail bytes).
-** Combining characters and East Asian Wide and Fullwidth characters are counted
-** as one, so this function does not calculate the effective "display width".
+** Return information about the next (single- or multi-byte) character in the
+** specified UTF-8 string: The number of UTF-8 code units (in this case: bytes)
+** and the decoded UTF-32 code point. Incomplete, ill-formed and overlong
+** sequences are consumed together as one invalid code point. The invalid lead
+** bytes 0xC0 to 0xC1 and 0xF5 to 0xF7 are allowed to initiate (ill-formed) 2-
+** and 4-byte sequences, respectively, the other invalid lead bytes 0xF8 to 0xFF
+** are treated as invalid 1-byte sequences (as lone trail bytes), all resulting
+** in one invalid code point. Invalid UTF-8 sequences encoding a non-scalar code
+** point (UTF-16 surrogates U+D800 to U+DFFF) are allowed.
 */
-int strlen_utf8(const char *zString, int lengthBytes){
-  int i;          /* Counted bytes. */
-  int lengthUTF8; /* Counted UTF-8 sequences. */
-#if 0
-  assert( lengthBytes>=0 );
-#endif
-  for(i=0, lengthUTF8=0; i<lengthBytes; i++, lengthUTF8++){
-    char c = zString[i];
-    int cchUTF8=1; /* Code units consumed. */
-    int maxUTF8=1; /* Expected sequence length. */
-    if( (c&0xe0)==0xc0 )maxUTF8=2;          /* UTF-8 lead byte 110vvvvv */
-    else if( (c&0xf0)==0xe0 )maxUTF8=3;     /* UTF-8 lead byte 1110vvvv */
-    else if( (c&0xf8)==0xf0 )maxUTF8=4;     /* UTF-8 lead byte 11110vvv */
-    while( cchUTF8<maxUTF8 &&
-            i<lengthBytes-1 &&
-            (zString[i+1]&0xc0)==0x80 ){    /* UTF-8 trail byte 10vvvvvv */
-      cchUTF8++;
-      i++;
-    }
+void char_info_utf8(
+  const char *z,
+  int *pCchUTF8,
+  int *pUtf32
+){
+  int i = 0;                              /* Counted bytes. */
+  int cchUTF8 = 1;                        /* Code units consumed. */
+  int maxUTF8 = 1;                        /* Expected sequence length. */
+  char c = z[i++];
+  if( (c&0x80)==0x00 ){                   /* 7-bit ASCII character. */
+    *pCchUTF8 = 1;
+    *pUtf32 = (int)z[0];
+    return;
   }
-  return lengthUTF8;
+  else if( (c&0xe0)==0xc0 ) maxUTF8 = 2;  /* UTF-8 lead byte 110vvvvv */
+  else if( (c&0xf0)==0xe0 ) maxUTF8 = 3;  /* UTF-8 lead byte 1110vvvv */
+  else if( (c&0xf8)==0xf0 ) maxUTF8 = 4;  /* UTF-8 lead byte 11110vvv */
+  while( cchUTF8<maxUTF8 &&
+          (z[i]&0xc0)==0x80 ){            /* UTF-8 trail byte 10vvvvvv */
+    cchUTF8++;
+    i++;
+  }
+  *pCchUTF8 = cchUTF8;
+  if( cchUTF8!=maxUTF8 ||                 /* Incomplete UTF-8 sequence. */
+      ( cchUTF8==1 && (c&0x80)==0x80 )){  /* Lone UTF-8 trail byte. */
+    *pUtf32 = 0xfffd;                     /* U+FFFD Replacement Character */
+#ifdef FOSSIL_DEBUG
+    assert( *pUtf32!=0xfffd );            /* Invalid UTF-8 sequence. */
+#endif
+    return;
+  }
+  switch( cchUTF8 ){
+    case 4:
+      *pUtf32 =
+        ( (z[0] & 0x0f)<<18 ) |
+        ( (z[1] & 0x3f)<<12 ) |
+        ( (z[2] & 0x3f)<< 6 ) |
+        ( (z[4] & 0x3f)<< 0 ) ;
+      break;
+    case 3:
+      *pUtf32 =
+        ( (z[0] & 0x0f)<<12 ) | 
+        ( (z[1] & 0x3f)<< 6 ) | 
+        ( (z[2] & 0x3f)<< 0 ) ;
+      break;
+    case 2:
+      *pUtf32 =
+        ( (z[0] & 0x1f)<< 6 ) | 
+        ( (z[1] & 0x3f)<< 0 ) ;
+      break;
+    default:
+      *pUtf32 = 0xfffd;                   /* U+FFFD Replacement Character */
+      break;
+  }
+#ifdef FOSSIL_DEBUG
+  assert(
+    *pUtf32>=0 && *pUtf32<=0x10ffff &&    /* Valid range U+0000 to U+10FFFF. */
+    *pUtf32<0xd800 && *pUtf32>0xdfff      /* Non-scalar (UTF-16 surrogates). */
+  );
+#endif
 }
 
 /*
@@ -209,7 +353,6 @@ static void comment_print_line(
 ){
   int index = 0, charCnt = 0, lineCnt = 0, maxChars, i;
   char zBuf[400]; int iBuf=0; /* Output buffer and counter. */
-  int cchUTF8, maxUTF8;       /* Helper variables to count UTF-8 sequences. */
   if( !zLine ) return;
   if( lineChars<=0 ) return;
 #if 0
@@ -232,6 +375,7 @@ static void comment_print_line(
   }
   maxChars = lineChars;
   for(;;){
+    int cchUTF8, utf32;
     int useChars = 1;
     char c = zLine[index];
     /* Flush the output buffer if there's no space left for at least one more
@@ -263,9 +407,9 @@ static void comment_print_line(
       charCnt = 0;
       useChars = 0;
     }else if( c=='\t' ){
-      int distUTF8;
-      int nextIndex = comment_next_space(zLine, index, &distUTF8);
-      if( nextIndex<=0 || distUTF8>maxChars ){
+      int sumWidth;
+      int nextIndex = comment_next_space(zLine, index, maxChars, &sumWidth);
+      if( nextIndex<=0 || sumWidth>maxChars ){
         break;
       }
       charCnt++;
@@ -275,9 +419,9 @@ static void comment_print_line(
         break;
       }
     }else if( wordBreak && fossil_isspace(c) ){
-      int distUTF8;
-      int nextIndex = comment_next_space(zLine, index, &distUTF8);
-      if( nextIndex<=0 || distUTF8>=maxChars ){
+      int sumWidth;
+      int nextIndex = comment_next_space(zLine, index, maxChars, &sumWidth);
+      if( nextIndex<=0 || sumWidth>=maxChars ){
         break;
       }
       charCnt++;
@@ -286,16 +430,20 @@ static void comment_print_line(
     }
     assert( c!='\n' || charCnt==0 );
     zBuf[iBuf++] = c;
-    /* Skip over UTF-8 sequences, see comment on strlen_utf8() for details. */
-    cchUTF8=1; /* Code units consumed. */
-    maxUTF8=1; /* Expected sequence length. */
-    if( (c&0xe0)==0xc0 )maxUTF8=2;          /* UTF-8 lead byte 110vvvvv */
-    else if( (c&0xf0)==0xe0 )maxUTF8=3;     /* UTF-8 lead byte 1110vvvv */
-    else if( (c&0xf8)==0xf0 )maxUTF8=4;     /* UTF-8 lead byte 11110vvv */
-    while( cchUTF8<maxUTF8 &&
-            (zLine[index]&0xc0)==0x80 ){    /* UTF-8 trail byte 10vvvvvv */
-      cchUTF8++;
-      zBuf[iBuf++] = zLine[index++];
+    char_info_utf8(&zLine[index-1],&cchUTF8,&utf32);
+    if( cchUTF8>1 ){
+      int wcwidth;
+      wcwidth = cli_wcwidth(utf32);
+      if( wcwidth>maxChars && lineChars>=wcwidth ){ /* rollback */
+        index--;
+        iBuf--;
+        zBuf[iBuf] = 0;
+        break;
+      }
+      for( ; cchUTF8>1; cchUTF8-- ){
+        zBuf[iBuf++] = zLine[index++];
+      }
+      useChars += wcwidth - 1;
     }
     maxChars -= useChars;
     if( maxChars<=0 ) break;
@@ -341,7 +489,6 @@ static int comment_print_legacy(
   char *zBuf;
   char zBuffer[400];
   int lineCnt = 0;
-  int cchUTF8, maxUTF8; /* Helper variables to count UTF-8 sequences. */
 
   if( width<0 ){
     comment_set_maxchars(indent, &maxChars);
@@ -367,21 +514,21 @@ static int comment_print_legacy(
       return lineCnt;
     }
     for(sk=si=i=k=kc=0; zText[i] && kc<maxChars; i++){
+      int cchUTF8, utf32;
       char c = zText[i];
       kc++; /* Count complete UTF-8 sequences. */
-      /* Skip over UTF-8 sequences, see comment on strlen_utf8() for details. */
-      cchUTF8=1; /* Code units consumed. */
-      maxUTF8=1; /* Expected sequence length. */
-      if( (c&0xe0)==0xc0 )maxUTF8=2;        /* UTF-8 lead byte 110vvvvv */
-      else if( (c&0xf0)==0xe0 )maxUTF8=3;   /* UTF-8 lead byte 1110vvvv */
-      else if( (c&0xf8)==0xf0 )maxUTF8=4;   /* UTF-8 lead byte 11110vvv */
-      if( maxUTF8>1 ){
-        zBuf[k++] = c;
-        while( cchUTF8<maxUTF8 &&
-                (zText[i+1]&0xc0)==0x80 ){  /* UTF-8 trail byte 10vvvvvv */
-          cchUTF8++;
+      char_info_utf8(&zText[i],&cchUTF8,&utf32);
+      if( cchUTF8>1 ){
+        int wcwidth;
+        wcwidth = cli_wcwidth(utf32);
+        if( kc+wcwidth-1>maxChars && maxChars>=wcwidth ){ /* rollback */
+          kc--;
+          break;
+        }
+        for( i--; cchUTF8>0; cchUTF8-- ){
           zBuf[k++] = zText[++i];
         }
+        kc += wcwidth - 1;
       }
       else if( fossil_isspace(c) ){
         si = i;
@@ -500,7 +647,9 @@ int comment_print(
     comment_print_line(zOrigText, zLine, indent, zLine>zText ? indent : 0,
                        maxChars, trimCrLf, trimSpace, wordBreak, origBreak,
                        &lineCnt, &zLine);
-    if( !zLine || !zLine[0] ) break;
+    if( zLine==0 ) break;
+    while( fossil_isspace(zLine[0]) ) zLine++;
+    if( zLine[0]==0 ) break;
   }
   return lineCnt;
 }
@@ -600,6 +749,7 @@ void test_comment_format(void){
   }else{
     indent = -1; /* automatic */
   }
+  verify_all_options();
   if( g.argc!=4 && g.argc!=5 ){
     usage("?OPTIONS? PREFIX TEXT ?ORIGTEXT?");
   }
