@@ -872,14 +872,15 @@ void help_page(void){
     }
   }else{
     int i;
+    const char *zWidth = "28ex";
     unsigned char occHelp[FOSSIL_MX_CMDIDX] = {0};   /* Help str occurrences */
     int bktHelp[FOSSIL_MX_CMDIDX][MX_HELP_DUP] = {{0}};/* Help str->commands */
     style_header("Help");
 
     cgi_printf("<a name='commands'></a>\n"
            "<h1>Available commands:</h1>\n"
-           "<div class=\"columns\" style=\"column-width: 12ex;\">\n"
-           "<ul>\n");
+           "<div class=\"columns\" style=\"column-width: %s;\">\n"
+           "<ul>\n",(zWidth));
     /* Fill in help string buckets */
     for(i=0; i<MX_COMMAND; i++){
       if(aCommand[i].eCmdFlags & CMDFLAG_HIDDEN) continue;
@@ -923,8 +924,8 @@ void help_page(void){
 
     cgi_printf("<a name='webpages'></a>\n"
            "<h1>Available web UI pages:</h1>\n"
-           "<div class=\"columns\" style=\"column-width: 18ex;\">\n"
-           "<ul>\n");
+           "<div class=\"columns\" style=\"column-width: %s;\">\n"
+           "<ul>\n",(zWidth));
     for(i=0; i<MX_COMMAND; i++){
       const char *z = aCommand[i].zName;
       if( '/'!=*z ) continue;
@@ -939,8 +940,8 @@ void help_page(void){
 
     cgi_printf("<a name='unsupported'></a>\n"
            "<h1>Unsupported commands:</h1>\n"
-           "<div class=\"columns\" style=\"column-width: 20ex;\">\n"
-           "<ul>\n");
+           "<div class=\"columns\" style=\"column-width: %s;\">\n"
+           "<ul>\n",(zWidth));
     for(i=0; i<MX_COMMAND; i++){
       const char *z = aCommand[i].zName;
       if( strncmp(z,"test",4)!=0 ) continue;
@@ -955,8 +956,8 @@ void help_page(void){
 
     cgi_printf("<a name='settings'></a>\n"
            "<h1>Settings:</h1>\n"
-           "<div class=\"columns\" style=\"column-width: 20ex;\">\n"
-           "<ul>\n");
+           "<div class=\"columns\" style=\"column-width: %s;\">\n"
+           "<ul>\n",(zWidth));
     for(i=0; i<MX_COMMAND; i++){
       const char *z = aCommand[i].zName;
       if( (aCommand[i].eCmdFlags & CMDFLAG_SETTING)==0 ) continue;
