@@ -156,6 +156,7 @@ static const struct {
   { "ipx",        3, "application/x-ipix"                },
   { "jad",        3, "text/vnd.sun.j2me.app-descriptor"  },
   { "jar",        3, "application/java-archive"          },
+  { "jp2",        3, "image/jp2"                         },
   { "jpe",        3, "image/jpeg"                        },
   { "jpeg",       4, "image/jpeg"                        },
   { "jpg",        3, "image/jpeg"                        },
@@ -255,6 +256,7 @@ static const struct {
   { "snd",        3, "audio/basic"                       },
   { "sol",        3, "application/solids"                },
   { "spl",        3, "application/x-futuresplash"        },
+  { "sql",        3, "application/sql"                   },
   { "src",        3, "application/x-wais-source"         },
   { "step",       4, "application/STEP"                  },
   { "stl",        3, "application/SLA"                   },
@@ -306,6 +308,8 @@ static const struct {
   { "xlw",        3, "application/vnd.ms-excel"          },
   { "xml",        3, "text/xml"                          },
   { "xpm",        3, "image/x-xpixmap"                   },
+  { "xsl",        3, "text/xml"                          },
+  { "xslt",       4, "text/xml"                          },
   { "xwd",        3, "image/x-xwindowdump"               },
   { "xyz",        3, "chemical/x-pdb"                    },
   { "zip",        3, "application/zip"                   },
