@@ -37,6 +37,8 @@
 # include <sys/utime.h>
 #else
 # include <sys/time.h>
+# include <pwd.h>
+# include <grp.h>
 #endif
 
 #if INTERFACE
@@ -233,6 +235,20 @@ int file_isfile(const char *zFilename, int eFType){
   return getStat(zFilename, eFType) ? 0 : S_ISREG(fx.fileStat.st_mode);
 }
 
+/*
+** Return TRUE if zFilename is a socket.
+*/
+int file_issocket(const char *zFilename){
+#ifdef _WIN32
+  return 0;
+#else
+  if( getStat(zFilename, ExtFILE) ){
+    return 0;  /* stat() failed.  Return false. */
+  }
+  return S_ISSOCK(fx.fileStat.st_mode);
+#endif
+}
+
 /*
 ** Create a symbolic link named zLinkFile that points to zTargetFile.
 **
@@ -717,6 +733,60 @@ void test_set_mtime(void){
   fossil_print("Set mtime of \"%s\" to %s (%lld)\n", zFile, zDate, iMTime);
 }
 
+/*
+** Change access permissions on a file.
+*/
+void file_set_mode(const char *zFN, int fd, const char *zMode, int bNoErr){
+#if !defined(_WIN32)
+  mode_t m;
+  char *zEnd = 0;
+  m = strtol(zMode, &zEnd, 0);
+  if( (zEnd[0] || fchmod(fd, m)) && !bNoErr ){
+    fossil_fatal("cannot change permissions on %s to \"%s\"",
+                 zFN, zMode);
+  }
+#endif
+}
+
+/* Change the owner of a file to zOwner.  zOwner can be of the form
+** USER:GROUP.
+*/
+void file_set_owner(const char *zFN, int fd, const char *zOwner){
+#if !defined(_WIN32)
+  const char *zGrp;
+  const char *zUsr = zOwner;
+  struct passwd *pw;
+  struct group *grp;
+  uid_t uid = -1;
+  gid_t gid = -1;
+  zGrp = strchr(zUsr, ':');
+  if( zGrp ){
+    int n = (int)(zGrp - zUsr);
+    zUsr = fossil_strndup(zUsr, n);
+    zGrp++;
+  }
+  pw = getpwnam(zUsr);
+  if( pw==0 ){
+    fossil_fatal("no such user: \"%s\"", zUsr);
+  }
+  uid = pw->pw_uid;
+  if( zGrp ){
+    grp = getgrnam(zGrp);
+    if( grp==0 ){
+      fossil_fatal("no such group: \"%s\"", zGrp);
+    }
+    gid = grp->gr_gid;
+  }
+  if( chown(zFN, uid, gid) ){
+    fossil_fatal("cannot change ownership of %s to %s",zFN, zOwner);
+  }
+  if( zOwner!=zUsr ){
+    fossil_free((char*)zUsr);
+  }
+#endif
+}
+
+
 /*
 ** Delete a file.
 **
@@ -1305,6 +1375,7 @@ char *file_canonical_name_dup(const char *zOrigName){
 ** in the preserved casing.  That's what this routine does.
 */
 char *file_case_preferred_name(const char *zDir, const char *zPath){
+#ifndef _WIN32 /* Call win32_file_case_preferred_name() on Windows. */
   DIR *d;
   int i;
   char *zResult = 0;
@@ -1340,6 +1411,9 @@ char *file_case_preferred_name(const char *zDir, const char *zPath){
   fossil_path_free(zNative);
   if( zResult==0 ) zResult = fossil_strdup(zPath);
   return zResult;
+#else /* !_WIN32 */
+  return win32_file_case_preferred_name(zDir,zPath);
+#endif /* !_WIN32 */
 }
 
 /*
@@ -1517,6 +1591,7 @@ static void emitFileStat(
   fossil_print("  file_mode(ExtFILE)     = 0%o\n", file_mode(zPath,ExtFILE));
   fossil_print("  file_isfile(ExtFILE)   = %d\n", file_isfile(zPath,ExtFILE));
   fossil_print("  file_isdir(ExtFILE)    = %d\n", file_isdir(zPath,ExtFILE));
+  fossil_print("  file_issocket()        = %d\n", file_issocket(zPath));
   if( reset ) resetStat();
   sqlite3_snprintf(sizeof(zBuf), zBuf, "%lld", file_size(zPath,RepoFILE));
   fossil_print("  file_size(RepoFILE)    = %s\n", zBuf);
@@ -1569,7 +1644,7 @@ void cmd_test_file_environment(void){
   if( zAllow ){
     g.allowSymlinks = !is_false(zAllow);
   }
-  if( zRoot==0 ) zRoot = g.zLocalRoot;
+  if( zRoot==0 ) zRoot = g.zLocalRoot==0 ? "" : g.zLocalRoot;
   fossil_print("db_allow_symlinks() = %d\n", db_allow_symlinks());
   fossil_print("local-root = [%s]\n", zRoot);
   for(i=2; i<g.argc; i++){
@@ -2412,6 +2487,9 @@ const char *file_cleanup_fullpath(const char *z){
 ** Count the number of objects (files and subdirectories) in a given
 ** directory.  Return the count.  Return -1 if the object is not a
 ** directory.
+**
+** This routine never counts the two "." and ".." special directory
+** entries, even if the provided glob would match them.
 */
 int file_directory_size(const char *zDir, const char *zGlob, int omitDotFiles){
   void *zNative;
@@ -2424,7 +2502,13 @@ int file_directory_size(const char *zDir, const char *zGlob, int omitDotFiles){
     n = 0;
     while( (pEntry=readdir(d))!=0 ){
       if( pEntry->d_name[0]==0 ) continue;
-      if( omitDotFiles && pEntry->d_name[0]=='.' ) continue;
+      if( pEntry->d_name[0]=='.' &&
+          (omitDotFiles
+           /* Skip the special "." and ".." entries. */
+           || pEntry->d_name[1]==0
+           || (pEntry->d_name[1]=='.' && pEntry->d_name[2]==0))){
+        continue;
+      }
       if( zGlob ){
         char *zUtf8 = fossil_path_to_utf8(pEntry->d_name);
         int rc = sqlite3_strglob(zGlob, zUtf8);
