@@ -188,7 +188,7 @@ void finfo_cmd(void){
     iLimit = zLimit ? atoi(zLimit) : -1;
     zOffset = find_option("offset",0,1);
     iOffset = zOffset ? atoi(zOffset) : 0;
-    iBrief = (find_option("brief","b",0) == 0);
+    iBrief = find_option("brief","b",0) != 0;
     if( iLimit==0 ){
       iLimit = -1;
     }
@@ -231,7 +231,7 @@ void finfo_cmd(void){
         iLimit, iOffset
     );
     blob_zero(&line);
-    if( iBrief ){
+    if( iBrief == 0 ){
       fossil_print("History for %s\n", blob_str(&fname));
     }
     while( db_step(&q)==SQLITE_ROW ){
@@ -243,7 +243,7 @@ void finfo_cmd(void){
       const char *zBr = db_column_text(&q, 5);
       char *zOut;
       if( zBr==0 ) zBr = "trunk";
-      if( iBrief ){
+      if( iBrief == 0 ){
         fossil_print("%s ", zDate);
         zOut = mprintf(
            "[%S] %s (user: %s, artifact: [%S], branch: %s)",
@@ -623,6 +623,7 @@ void finfo_page(void){
     int gidx;
     char zTime[10];
     int nParent = 0;
+    int bIsModified = 0;
     GraphRowId aParent[GR_MAX_RAIL];
 
     db_bind_int(&qparent, ":fid", frid);
@@ -665,6 +666,9 @@ void finfo_page(void){
     }else{
       cgi_printf("<td class=\"timeline%sCell\">\n",(zStyle));
     }
+    if( zPUuid && zUuid && fossil_strcmp(zPUuid, zUuid)!=0 ){
+      bIsModified = 1;
+    }
     if( tmFlags & TIMELINE_COMPACT ){
       cgi_printf("<span class='timelineCompactComment' data-id='%d'>\n",(frid));
     }else{
@@ -672,7 +676,11 @@ void finfo_page(void){
       if( pfnid ){
         char *zPrevName = db_text(0,"SELECT name FROM filename WHERE fnid=%d",
                                    pfnid);
-        cgi_printf("<b>Renamed</b> %h &rarr; %h.\n",(zPrevName),(zFName));
+        if( bIsModified ){
+          cgi_printf("<b>Renamed and modified</b> %h &rarr; %h.\n",(zPrevName),(zFName));
+        }else{
+          cgi_printf("<b>Renamed</b> %h &rarr; %h.\n",(zPrevName),(zFName));
+        }
         fossil_free(zPrevName);
       }
       if( zUuid && ridTo==0 && nParent==0 ){
@@ -759,7 +767,7 @@ void finfo_page(void){
              "%z\n"
              "[blame]</a>\n"
              "%z[check-ins&nbsp;using]</a>\n",(frid),(href("%R/annotate?filename=%h&checkin=%s",z,zCkin)),(href("%R/blame?filename=%h&checkin=%s",z,zCkin)),(href("%R/timeline?uf=%!S",zUuid)));
-      if( fpid>0 ){
+      if( fpid>0 && bIsModified!=0 ){
         cgi_printf("%z[diff]</a>\n",(href("%R/fdiff?v1=%!S&v2=%!S",zPUuid,zUuid)));
       }
       if( fileedit_is_editable(zFName) ){
