@@ -1400,8 +1400,9 @@ static void forum_post_widget(
   }
   cgi_printf("%zMarkup style</a>:\n",(href("%R/markup_help")));
   mimetype_option_menu(zMimetype, "mimetype");
-  cgi_printf("<br><textarea aria-label=\"Content:\" name=\"content\" class=\"wikiedit\" "
-         "cols=\"80\" rows=\"25\" wrap=\"virtual\">%h</textarea><br>\n",(zContent));
+  cgi_printf("<div class=\"forum-editor-widget\">\n"
+         "<textarea aria-label=\"Content:\" name=\"content\" class=\"wikiedit\" "
+         "cols=\"80\" rows=\"25\" wrap=\"virtual\">%h</textarea></div>\n",(zContent));
 }
 
 /*
@@ -1523,7 +1524,7 @@ static void forum_render_debug_options(void){
            "<br><label><input type=\"checkbox\" name=\"showqp\" %s> "
            "Show query parameters</label>\n"
            "<br><label><input type=\"checkbox\" name=\"fpsilent\" %s> "
-           "Do not sent notification emails</label>\n"
+           "Do not send notification emails</label>\n"
            "</div>\n",(PCK("dryrun")),(PCK("domod")),(PCK("showqp")),(PCK("fpsilent")));
   }
 }
