@@ -797,27 +797,37 @@ LOCAL void search_fullscan(
     char *zDocGlob = db_get("doc-glob","");
     char *zDocBr = db_get("doc-branch","trunk");
     if( zDocGlob && zDocGlob[0] && zDocBr && zDocBr[0] ){
+      Glob * pGlob = glob_create(zDocBr)
+        /* We're misusing a Glob as a list of comma-/space-delimited
+        ** tokens. We're not actually doing glob matches here. */;
+      int i;
       db_multi_exec(
         "CREATE VIRTUAL TABLE IF NOT EXISTS temp.foci USING files_of_checkin;"
       );
-      db_multi_exec(
-        "INSERT INTO x(label,url,score,id,date,snip)"
-        "  SELECT printf('Document: %%s',title('d',blob.rid,foci.filename)),"
-        "         printf('/doc/%T/%%s',foci.filename),"
-        "         search_score(),"
-        "         'd'||blob.rid,"
-        "         (SELECT datetime(event.mtime) FROM event"
-        "            WHERE objid=symbolic_name_to_rid('trunk')),"
-        "         search_snippet()"
-        "    FROM foci CROSS JOIN blob"
-        "   WHERE checkinID=symbolic_name_to_rid('trunk')"
-        "     AND blob.uuid=foci.uuid"
-        "     AND search_match(title('d',blob.rid,foci.filename),"
-        "                      body('d',blob.rid,foci.filename))"
-        "     AND %z",
-        zDocBr, glob_expr("foci.filename", zDocGlob)
-      );
+      for( i = 0; i < pGlob->nPattern; ++i ){
+        const char * zBranch = pGlob->azPattern[i];
+        db_multi_exec(
+          "INSERT INTO x(label,url,score,id,date,snip)"
+          "  SELECT printf('Document: %%s',title('d',blob.rid,foci.filename)),"
+          "         printf('/doc/%T/%%s',foci.filename),"
+          "         search_score(),"
+          "         'd'||blob.rid,"
+          "         (SELECT datetime(event.mtime) FROM event"
+          "            WHERE objid=symbolic_name_to_rid(%Q)),"
+          "         search_snippet()"
+          "    FROM foci CROSS JOIN blob"
+          "   WHERE checkinID=symbolic_name_to_rid(%Q)"
+          "     AND blob.uuid=foci.uuid"
+          "     AND search_match(title('d',blob.rid,foci.filename),"
+          "                      body('d',blob.rid,foci.filename))"
+          "     AND %z",
+          zBranch, zBranch, zBranch, glob_expr("foci.filename", zDocGlob)
+        );
+      }
+      glob_free(pGlob);
     }
+    fossil_free(zDocGlob);
+    fossil_free(zDocBr);
   }
   if( (srchFlags & SRCH_WIKI)!=0 ){
     db_multi_exec(
@@ -979,6 +989,29 @@ static void search_rank_sqlfunc(
 #endif
 }
 
+/*
+** Expects a search pattern string. Makes a copy of the string,
+** replaces all non-alphanum ASCII characters with a space, and
+** lower-cases all upper-case ASCII characters. The intent is to avoid
+** causing errors in FTS5 searches with inputs which contain AND, OR,
+** and symbols like #. The caller is responsible for passing the
+** result to fossil_free().
+*/
+char *search_simplify_pattern(const char * zPattern){
+  char *zPat = mprintf("%s",zPattern);
+  int i;
+  for(i=0; zPat[i]; i++){
+    if( (zPat[i]&0x80)==0 && !fossil_isalnum(zPat[i]) ) zPat[i] = ' ';
+    if( fossil_isupper(zPat[i]) ) zPat[i] = fossil_tolower(zPat[i]);
+  }
+  for(i--; i>=0 && zPat[i]==' '; i--){}
+  if( i<0 ){
+    fossil_free(zPat);
+    zPat = mprintf("\"\"");
+  }
+  return zPat;
+}
+
 /*
 ** When this routine is called, there already exists a table
 **
@@ -1000,20 +1033,12 @@ LOCAL void search_indexed(
   unsigned int srchFlags      /* What to search over */
 ){
   Blob sql;
-  char *zPat = mprintf("%s",zPattern);
-  int i;
+  char *zPat;
   static const char *zSnippetCall;
   if( srchFlags==0 ) return;
   sqlite3_create_function(g.db, "rank", 1, SQLITE_UTF8|SQLITE_INNOCUOUS, 0,
      search_rank_sqlfunc, 0, 0);
-  for(i=0; zPat[i]; i++){
-    if( (zPat[i]&0x80)==0 && !fossil_isalnum(zPat[i]) ) zPat[i] = ' ';
-  }
-  for(i--; i>=0 && zPat[i]==' '; i--){}
-  if( i<0 ){
-    fossil_free(zPat);
-    zPat = mprintf("\"\"");
-  }
+  zPat = search_simplify_pattern(zPattern);
   blob_init(&sql, 0, 0);
   if( search_index_type(0)==4 ){
     /* If this repo is still using the legacy FTS4 search index, then
@@ -1639,6 +1664,7 @@ static const char zFtsDrop[] =
 "DROP TABLE IF EXISTS repository.ftsidx;\n"
 "DROP VIEW IF EXISTS repository.ftscontent;\n"
 "DROP TABLE IF EXISTS repository.ftsdocs;\n"
+"DROP TABLE IF EXISTS repository.chatfts1;\n"
 ;
 
 #if INTERFACE
@@ -1684,6 +1710,21 @@ int search_tokenizer_type(int bRecheck){
   return iFtsTokenizer;
 }
 
+/*
+** Returns a string in the form ",tokenize=X", where X is the string
+** counterpart of the given FTS5TOK_xyz value.  Returns "" if tokType
+** does not correspond to a known FTS5 tokenizer.
+*/
+const char * search_tokenize_arg_for_type(int tokType){
+  switch( tokType ){
+    case FTS5TOK_PORTER: return ",tokenize=porter";
+    case FTS5TOK_UNICODE61: return ",tokenize=unicode61";
+    case FTS5TOK_TRIGRAM: return ",tokenize=trigram";
+    case FTS5TOK_NONE:
+    default: return "";
+  }
+}
+
 /*
 ** Returns a string value suitable for use as the search-tokenizer
 ** setting's value, depending on the value of z. If z is 0 then the
@@ -1728,14 +1769,9 @@ void search_set_tokenizer(const char *zName){
 */
 static int searchIdxExists = -1;
 void search_create_index(void){
-  const int useTokenizer = search_tokenizer_type(0);
-  const char *zExtra;
-  switch(useTokenizer){
-    case FTS5TOK_PORTER: zExtra = ",tokenize=porter"; break;
-    case FTS5TOK_UNICODE61: zExtra = ",tokenize=unicode61"; break;
-    case FTS5TOK_TRIGRAM: zExtra = ",tokenize=trigram"; break;
-    default: zExtra = ""; break;
-  }
+  const char *zExtra =
+    search_tokenize_arg_for_type(search_tokenizer_type(0));
+  assert( zExtra );
   search_sql_setup(g.db);
   db_multi_exec(zFtsSchema/*works-like:"%s"*/, zExtra/*safe-for-%s*/);
   searchIdxExists = 1;
@@ -1855,55 +1891,64 @@ void search_doc_touch(char cType, int rid, const char *zName){
 ** changed.
 */
 static void search_update_doc_index(void){
-  const char *zDocBr = db_get("doc-branch","trunk");
-  int ckid = zDocBr ? symbolic_name_to_rid(zDocBr,"ci") : 0;
-  double rTime;
-  if( ckid==0 ) return;
-  if( !db_exists("SELECT 1 FROM ftsdocs WHERE type='c' AND rid=%d"
-                 "   AND NOT idxed", ckid) ) return;
-
-  /* If we get this far, it means that changes to 'd' entries are
-  ** required. */
-  rTime = db_double(0.0, "SELECT mtime FROM event WHERE objid=%d", ckid);
+  const char *zDocBranches = db_get("doc-branch","trunk");
+  int i;
+  Glob * pGlob = glob_create(zDocBranches)
+    /* We're misusing a Glob as a list of comma-/space-delimited
+    ** tokens. We're not actually doing glob matches here. */;
+  if( !pGlob ) return;
   db_multi_exec(
     "CREATE TEMP TABLE current_docs(rid INTEGER PRIMARY KEY, name);"
     "CREATE VIRTUAL TABLE IF NOT EXISTS temp.foci USING files_of_checkin;"
-    "INSERT OR IGNORE INTO current_docs(rid, name)"
-    "  SELECT blob.rid, foci.filename FROM foci, blob"
-    "   WHERE foci.checkinID=%d AND blob.uuid=foci.uuid"
-    "     AND %z",
-    ckid, glob_expr("foci.filename", db_get("doc-glob",""))
-  );
-  db_multi_exec(
-    "DELETE FROM ftsidx WHERE rowid IN"
-    "  (SELECT rowid FROM ftsdocs WHERE type='d'"
-    "      AND rid NOT IN (SELECT rid FROM current_docs))"
-  );
-  db_multi_exec(
-    "DELETE FROM ftsdocs WHERE type='d'"
-    "      AND rid NOT IN (SELECT rid FROM current_docs)"
-  );
-  db_multi_exec(
-    "INSERT OR IGNORE INTO ftsdocs(type,rid,name,idxed,label,bx,url,mtime)"
-    "  SELECT 'd', rid, name, 0,"
-    "         title('d',rid,name),"
-    "         body('d',rid,name),"
-    "         printf('/doc/%T/%%s',urlencode(name)),"
-    "         %.17g"
-    " FROM current_docs",
-    zDocBr, rTime
-  );
-  db_multi_exec(
-    "INSERT INTO ftsidx(rowid,title,body)"
-    "  SELECT rowid, label, bx FROM ftsdocs WHERE type='d' AND NOT idxed"
-  );
-  db_multi_exec(
-    "UPDATE ftsdocs SET"
-    "  idxed=1,"
-    "  bx=NULL,"
-    "  label='Document: '||label"
-    " WHERE type='d' AND NOT idxed"
   );
+  for( i = 0; i < pGlob->nPattern; ++i ){
+    const char *zDocBr = pGlob->azPattern[i];
+    int ckid = symbolic_name_to_rid(zDocBr,"ci");
+    double rTime;
+    if( !db_exists("SELECT 1 FROM ftsdocs WHERE type='c' AND rid=%d"
+                   "   AND NOT idxed", ckid) ) continue;
+    /* If we get this far, it means that changes to 'd' entries are
+    ** required. */
+    rTime = db_double(0.0, "SELECT mtime FROM event WHERE objid=%d", ckid);
+    db_multi_exec(
+      "INSERT OR IGNORE INTO current_docs(rid, name)"
+      "  SELECT blob.rid, foci.filename FROM foci, blob"
+      "   WHERE foci.checkinID=%d AND blob.uuid=foci.uuid"
+      "     AND %z",
+      ckid, glob_expr("foci.filename", db_get("doc-glob",""))
+    );
+    db_multi_exec(
+      "DELETE FROM ftsidx WHERE rowid IN"
+      "  (SELECT rowid FROM ftsdocs WHERE type='d'"
+      "      AND rid NOT IN (SELECT rid FROM current_docs))"
+    );
+    db_multi_exec(
+      "DELETE FROM ftsdocs WHERE type='d'"
+      "      AND rid NOT IN (SELECT rid FROM current_docs)"
+    );
+    db_multi_exec(
+      "INSERT OR IGNORE INTO ftsdocs(type,rid,name,idxed,label,bx,url,mtime)"
+      "  SELECT 'd', rid, name, 0,"
+      "         title('d',rid,name),"
+      "         body('d',rid,name),"
+      "         printf('/doc/%T/%%s',urlencode(name)),"
+      "         %.17g"
+      " FROM current_docs",
+      zDocBr, rTime
+    );
+    db_multi_exec(
+      "INSERT INTO ftsidx(rowid,title,body)"
+      "  SELECT rowid, label, bx FROM ftsdocs WHERE type='d' AND NOT idxed"
+    );
+    db_multi_exec(
+      "UPDATE ftsdocs SET"
+      "  idxed=1,"
+      "  bx=NULL,"
+      "  label='Document: '||label"
+      " WHERE type='d' AND NOT idxed"
+    );
+  }
+  glob_free(pGlob);
 }
 
 /*
@@ -2059,6 +2104,9 @@ void search_rebuild_index(void){
   search_create_index();
   search_fill_index();
   search_update_index(search_restrict(SRCH_ALL));
+  if( db_table_exists("repository","chat") ){
+    chat_rebuild_index(1);
+  }
   fossil_print(" done\n");
 }
 
