@@ -579,7 +579,7 @@ void secaudit0_page(void){
            "filesystem is mounted within the jail, so that the load average\n"
            "can be obtained from the /proc/loadavg file.\n");
   }else {
-    double r = atof(db_get("max-loadavg", 0));
+    double r = atof(db_get("max-loadavg", "0.0"));
     if( r<=0.0 ){
       cgi_printf("<li><p>\n"
              "Load average limiting is turned off.  This can cause the server\n"
@@ -696,6 +696,33 @@ void secaudit0_page(void){
     cgi_printf("</li>\n");
   }
 
+  cgi_printf("<li><p>Robot Defenses:\n"
+         "<ol type=\"a\">\n");
+  switch( db_get_int("auto-hyperlink",1) ){
+    default:
+       cgi_printf("<li> No auto-enable of hyperlinks.\n");
+       break;
+    case 1:
+       cgi_printf("<li> Hyperlinks auto-enabled based on UserAgent and Javascript.\n");
+       break;
+    case 2:
+       cgi_printf("<li> Hyperlinks auto-enabled based on UserAgent only.\n");
+       break;
+  }
+  z = db_get("max-loadavg",0);
+  if( z && atof(z)>0.0 ){
+    cgi_printf("<li> Maximum load average for expensive requests: %h;\n",(z));
+  }else{
+    cgi_printf("<li> No limits on the load average\n");
+  }
+  z = db_get("robot-restrict",0);
+  if( z==0 ){
+    cgi_printf("<li> No complex-request constraints on robots\n");
+  }else{
+    cgi_printf("<li> Complex requests limited for pages matching: %h\n",(z));
+  }
+  cgi_printf("</ol>\n");
+
   blob_init(&cmd, 0, 0);
   for(i=0; g.argvOrig[i]!=0; i++){
     blob_append_escaped_arg(&cmd, g.argvOrig[i], 0);
@@ -806,6 +833,7 @@ void errorlog_page(void){
   style_submenu_element("Refresh", "%R/errorlog");
   style_submenu_element("Log-Menu", "%R/setup-logmenu");
   style_submenu_element("Panics", "%R/paniclog");
+  style_submenu_element("Non-Hacks", "%R/hacklog?not");
 
   if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
     no_error_log_available();
@@ -926,3 +954,71 @@ void paniclog_page(void){
   cgi_printf("</pre>\n");
   style_finish_page();
 }
+
+/*
+** WEBPAGE: hacklog
+**
+** Scan the error log for "possible hack attempt" entries  Show hack
+** attempt messages only, omitting all others.  Or if the "not" query
+** parameter is present, show only messages that are not hack attempts.
+*/
+void hacklog_page(void){
+  i64 szFile;
+  char *zLog;
+  FILE *in;
+  int bOutput = 0;
+  int prevWasTime = 0;
+  int isNot = P("not")!=0;
+  char z[10000];
+  char zTime[10000];
+
+  login_check_credentials();
+  if( !g.perm.Admin ){
+    login_needed(0);
+    return;
+  }
+  style_header("Server Hack Log");
+  style_submenu_element("Log-Menu", "%R/setup-logmenu");
+
+  if( g.zErrlog==0 || fossil_strcmp(g.zErrlog,"-")==0 ){
+    no_error_log_available();
+    style_finish_page();
+    return;
+  }
+  in = fossil_fopen(g.zErrlog, "rb");
+  if( in==0 ){
+    cgi_printf("<p class='generalError'>Unable to open that file for reading!</p>\n");
+    style_finish_page();
+    return;
+  }
+  szFile = file_size(g.zErrlog, ExtFILE);
+  zLog = file_canonical_name_dup(g.zErrlog);
+  cgi_printf("%s messages contained within the %lld-byte \n"
+         "<a href=\"%R/errorlog?all\">error log</a> found at\n"
+         "\"%h\".\n",(isNot?"Non-hack":"Hack"),(szFile),(zLog));
+  fossil_free(zLog);
+  cgi_printf("<hr>\n"
+         "<pre>\n");
+  while( fgets(z, sizeof(z), in) ){
+    if( prevWasTime 
+     && ((strncmp(z,"possible hack attempt - 418 ", 27)==0) ^ isNot)
+    ){
+      cgi_printf("%h",(zTime));
+      bOutput = 1;
+    }
+    if( strncmp(z, "--------", 8)==0 ){
+      size_t n = strlen(z);
+      memcpy(zTime, z, n+1);
+      prevWasTime = 1;
+      bOutput = 0;
+    }else{
+      prevWasTime = 0;
+    }
+    if( bOutput ){
+      cgi_printf("%h",(z));
+    }
+  }
+  fclose(in);
+  cgi_printf("</pre>\n");
+  style_finish_page();
+}
