@@ -239,6 +239,15 @@ void setup_logmenu_page(void){
     "in the \"accesslog\" table of the respository.\n"
   );
 
+  setup_menu_entry("Hack Log", "hacklog",
+    "All 418 hack attempts"
+  );
+
+  setup_menu_entry("Non-Hack Log", "hacklog?not",
+    "All log messages that are not hack attempts"
+  );
+
+
   cgi_printf("</table>\n");
   style_finish_page();
 }
@@ -484,6 +493,21 @@ void setup_robots(void){
          "might not work inside a chroot() jail.\n"
          "(Property: \"max-loadavg\")</p>\n");
 
+  cgi_printf("<hr>\n"
+         "<p><b>Do not allow robots to make complex requests\n"
+         "against the following pages.</b>\n"
+         "<p> A \"complex request\" is an HTTP request that has one or more query\n"
+         "parameters. Some robots will spend hours juggling around query parameters\n"
+         "or even forging fake query parameters in an effort to discover new\n"
+         "behavior or to find an SQL injection opportunity or similar.  This can\n"
+         "waste hours of CPU time and gigabytes of bandwidth on the server.  A\n"
+         "suggested value for this setting is:\n"
+         "\"<tt>timeline,*diff,vpatch,annotate,blame,praise,dir,tree</tt>\".\n"
+         "(Property: robot-restrict)\n"
+         "<p>\n");
+  textarea_attribute("", 2, 80,
+      "robot-restrict", "rbrestrict", "", 0);
+
   cgi_printf("<hr>\n"
          "<p><input type=\"submit\"  name=\"submit\" value=\"Apply Changes\"></p>\n"
          "</div></form>\n");
@@ -739,6 +763,7 @@ void setup_access(void){
 void setup_login_group(void){
   const char *zGroup;
   char *zErrMsg = 0;
+  Stmt q;
   Blob fullName;
   char *zSelfRepo;
   const char *zRepo = PD("repo", "");
@@ -758,6 +783,8 @@ void setup_login_group(void){
     login_group_join(zRepo, 1, zLogin, zPw, zNewName, &zErrMsg);
   }else if( P("leave") ){
     login_group_leave(&zErrMsg);
+  }else if( P("rotate") ){
+    captcha_secret_rotate();
   }
   style_set_current_feature("setup");
   style_header("Login Group Configuration");
@@ -800,7 +827,6 @@ void setup_login_group(void){
            "<input type=\"submit\" value=\"Join\" name=\"join\"></td></tr>\n"
            "</table></blockquote></div></form>\n",(zRepo),(zLogin),(zNewName));
   }else{
-    Stmt q;
     int n = 0;
     cgi_printf("<p>This repository (in the file \"%h\")\n"
            "is currently part of the \"<b>%h</b>\" login group.\n"
@@ -828,31 +854,74 @@ void setup_login_group(void){
            "\n"
            "<p><form action=\"%R/setup_login_group\" method=\"post\"><div>\n");
     login_insert_csrf_secret();
-    cgi_printf("To leave this login group press\n"
+    cgi_printf("<p>To leave this login group press:\n"
            "<input type=\"submit\" value=\"Leave Login Group\" name=\"leave\">\n"
-           "</form></p>\n"
-           "<hr><h2>Implementation Details</h2>\n"
-           "<p>The following are fields from the CONFIG table related to login-groups,\n"
-           "provided here for instructional and debugging purposes:</p>\n"
-           "<table border='1' class='sortable' data-column-types='ttt' "
-           "data-init-sort='1'>\n"
-           "<thead><tr>\n"
-           "<th>Config.Name<th>Config.Value<th>Config.mtime</tr>\n"
-           "</thead><tbody>\n");
-    db_prepare(&q, "SELECT name, value, datetime(mtime,'unixepoch') FROM config"
-                   " WHERE name GLOB 'peer-*'"
-                   "    OR name GLOB 'project-*'"
-                   "    OR name GLOB 'login-group-*'"
-                   " ORDER BY name");
-    while( db_step(&q)==SQLITE_ROW ){
-      cgi_printf("<tr><td>%h</td>\n"
-             "<td>%h</td>\n"
-             "<td>%h</td></tr>\n",(db_column_text(&q,0)),(db_column_text(&q,1)),(db_column_text(&q,2)));
-    }
-    db_finalize(&q);
-    cgi_printf("</tbody></table>\n");
-    style_table_sorter();
+           "<p>Setting a common captcha-secret on all repositories in the login-group\n"
+           "allows anonymous logins for one repository in the login group to be used\n"
+           "by all other repositories of the group within the same domain.  Warning:\n"
+           "If a captcha dialog was painted before setting the common captcha-secret\n"
+           "and the \"Speak password for 'anonymous'\" button is pressed afterwards,\n"
+           "the spoken text will be incorrect.\n"
+           "<input type=\"submit\" name=\"rotate\" value=\"Set common captcha-secret\">\n"
+           "</form></p>\n");
   }
+  cgi_printf("<hr><h2>Implementation Details</h2>\n"
+         "<p>The following are fields from the CONFIG table related to login-groups.\n"
+         "</p>\n"
+         "<table border='1' cellspacing=\"0\" cellpadding=\"4\""
+         "class='sortable' data-column-types='ttt' data-init-sort='1'>\n"
+         "<thead><tr>\n"
+         "<th>Config.Name<th>Config.Value<th>Config.mtime</tr>\n"
+         "</thead><tbody>\n");
+  db_prepare(&q, "SELECT name, value, datetime(mtime,'unixepoch') FROM config"
+                 " WHERE name GLOB 'peer-*'"
+                 "    OR name GLOB 'project-*'"
+                 "    OR name GLOB 'login-group-*'"
+                 " ORDER BY name");
+  while( db_step(&q)==SQLITE_ROW ){
+    cgi_printf("<tr><td>%h</td>\n"
+           "<td>%h</td>\n"
+           "<td>%h</td></tr>\n",(db_column_text(&q,0)),(db_column_text(&q,1)),(db_column_text(&q,2)));
+  }
+  db_finalize(&q);
+  cgi_printf("</tbody></table>\n"
+         "<h2>Interpretation</h2>\n"
+         "<ul>\n"
+         "<li><p><b>login-group-code</b> &rarr;\n"
+         "A random code assigned to each login-group.  The login-group-code is\n"
+         "a unique identifier for the login-group.\n"
+         "\n"
+         "<li><p><b>login-group-name</b> &rarr;\n"
+         "The human-readable name of the login-group.\n"
+         "\n"
+         "<li><p><b>project-code</b> &rarr;\n"
+         "A random code assigned to each project.  The project-code is\n"
+         "a unique identifier for the project.  Multiple repositories can share\n"
+         "the same project-code.  When two or more repositories have the same\n"
+         "project code, that mean those repositories are clones of each other.\n"
+         "Repositories are only able to sync if they share the same project-code.\n"
+         "\n"
+         "<li><p><b>project-description</b> &rarr;\n"
+         "A description of project in this repository.  This is a verbose form\n"
+         "of project-name.  This description can be edited in the second entry\n"
+         "box on the <a href=\"./setup_config\">Setup/Configuration page</a>.\n"
+         "\n"
+         "<li><p><b>project-name</b> &rarr;\n"
+         "The human-readable name for the project.  The project-name can be\n"
+         "modified in the first entry on the \n"
+         "<a href=\"./setup_config\">Setup/Configuration page</a>.\n"
+         "\n"
+         "<li><p><b>peer-repo-<i>CODE</i></b> &rarr;\n"
+         "<i>CODE</i> is 16-character prefix of the project-code for another\n"
+         "repository that is part of the same login-group.  The value is the\n"
+         "filename for the peer repository.\n"
+         "\n"
+         "<li><p><b>peer-name-<i>CODE</i></b> &rarr;\n"
+         "<i>CODE</i> is 16-character prefix of the project-code for another\n"
+         "repository that is part of the same login-group.  The value is\n"
+         "project-name value for the other repository.\n"
+         "</ul>\n");
+  style_table_sorter();
   style_finish_page();
 }
 
@@ -2121,10 +2190,11 @@ void page_srchsetup(){
          "<td>Search nothing. (Disables document search).</tr>\n"
          "</table>\n"
          "<hr>\n");
-  entry_attribute("Document Branch", 20, "doc-branch", "db", "trunk", 0);
+  entry_attribute("Document Branches", 20, "doc-branch", "db", "trunk", 0);
   cgi_printf("<p>When searching documents, use the versions of the files found at the\n"
-         "type of the \"Document Branch\" branch.  Recommended value: \"trunk\".\n"
-         "Document search is disabled if blank.\n"
+         "type of the \"Document Branches\" branch.  Recommended value: \"trunk\".\n"
+         "Document search is disabled if blank. It may be a list of branch names\n"
+         "separated by spaces and/or commas.\n"
          "<hr>\n");
   onoff_attribute("Search Check-in Comments", "search-ci", "sc", 0, 0);
   cgi_printf("<br>\n");
