@@ -124,10 +124,10 @@ void shun_page(void){
       for( p = zUuid ; *p ; p += strlen(p)+1 ){
         cgi_printf("%s<br>\n",(p));
       }
-      cgi_printf("will no longer be shunned.  But they may not exist in the repository.\n"
-             "It may be necessary to rebuild the repository using the\n"
-             "<b>fossil rebuild</b> command-line before the artifact content\n"
-             "can pulled in from other repositories.</p>\n");
+      cgi_printf("will no longer be shunned but they may not exist in the repository.\n"
+             "It may be necessary to rebuild the repository\n"
+             "before the artifact content can be pulled in\n"
+             "from other repositories.</p>\n");
     }
   }
   if( zUuid && P("add") && cgi_csrf_safe(2) ){
