@@ -339,6 +339,7 @@ static void xfer_accept_unversioned_file(Xfer *pXfer, int isWriter){
    || (!blob_eq(pHash,"-") && !blob_is_hname(pHash))
    || !blob_is_int(&pXfer->aToken[4], &sz)
    || !blob_is_int(&pXfer->aToken[5], &flags)
+   || (mtime<0 || sz<0 || flags<0)
   ){
     blob_appendf(&pXfer->err, "malformed uvfile line");
     return;
@@ -1123,6 +1124,20 @@ const char *xfer_ticket_code(void){
   return db_get("xfer-ticket-script", 0);
 }
 
+/*
+** Reset the CGI content, roll back any pending db transaction, and
+** emit an "error" xfer message. The message text gets fossil-encoded
+** by this function. This is only intended for use with
+** fail-fast/fatal errors, not ones which can be skipped over.
+*/
+static void xfer_fatal_error(const char *zMsg){
+  cgi_reset_content();
+  if( db_transaction_nesting_depth()>0 ){
+    db_rollback_transaction();
+  }
+  cgi_printf("error %F\n",(zMsg));
+}
+
 /*
 ** Run the specified TH1 script, if any, and returns 1 on error.
 */
@@ -1464,6 +1479,10 @@ void page_xfer(void){
           cgi_set_content_type("application/x-fossil-uncompressed");
         }
         blob_is_int(&xfer.aToken[2], &seqno);
+        if( seqno<=0 ){
+          xfer_fatal_error("invalid clone sequence number");
+          return;
+        }
         max = db_int(0, "SELECT max(rid) FROM blob");
         while( xfer.mxSend>(int)blob_size(xfer.pOut) && seqno<=max){
           if( time(NULL) >= xfer.maxTime ) break;
@@ -1535,6 +1554,10 @@ void page_xfer(void){
         && blob_is_int(&xfer.aToken[2], &size) ){
       const char *zName = blob_str(&xfer.aToken[1]);
       Blob content;
+      if( size<0 ){
+        xfer_fatal_error("invalid config record");
+        return;
+      }
       blob_zero(&content);
       blob_extract(xfer.pIn, size, &content);
       if( !g.perm.Admin ){
@@ -1847,7 +1870,7 @@ void page_xfer(void){
   */
   zNow = db_text(0, "SELECT strftime('%%Y-%%m-%%dT%%H:%%M:%%S', 'now')");
   cgi_printf("# timestamp %s errors %d\n",(zNow),(nErr));
-  free(zNow);
+  fossil_free(zNow);
 
   db_commit_transaction();
   configure_rebuild();
@@ -1992,6 +2015,7 @@ int client_sync(
   const char *zCkinLock;  /* Name of check-in to lock.  NULL for none */
   const char *zClientId;  /* A unique identifier for this check-out */
   unsigned int mHttpFlags;/* Flags for the http_exchange() subsystem */
+  const int bOutIsTty = fossil_isatty(fossil_fileno(stdout));
 
   if( pnRcvd ) *pnRcvd = 0;
   if( db_get_boolean("dont-push", 0) ) syncFlags &= ~SYNC_PUSH;
@@ -2307,8 +2331,10 @@ int client_sync(
     }else{
       nRoundtrip++;
       nArtifactSent += xfer.nFileSent + xfer.nDeltaSent;
-      fossil_print(zBriefFormat /*works-like:"%d%d%d"*/,
-                   nRoundtrip, nArtifactSent, nArtifactRcvd);
+      if( bOutIsTty!=0 ){
+        fossil_print(zBriefFormat /*works-like:"%d%d%d"*/,
+                     nRoundtrip, nArtifactSent, nArtifactRcvd);
+      }
     }
     nCardSent = 0;
     nCardRcvd = 0;
@@ -2487,6 +2513,10 @@ int client_sync(
         const char *zName = blob_str(&xfer.aToken[1]);
         const char *zHash = blob_str(&xfer.aToken[3]);
         int iStatus;
+        if( mtime<0 || size<0 ){
+          xfer_fatal_error("invalid uvigot");
+          return ++nErr;
+        }
         iStatus = unversioned_status(zName, mtime, zHash);
         if( (syncFlags & SYNC_UV_REVERT)!=0 ){
           if( iStatus==4 ) iStatus = 2;
@@ -2573,6 +2603,10 @@ int client_sync(
           && blob_is_int(&xfer.aToken[2], &size) ){
         const char *zName = blob_str(&xfer.aToken[1]);
         Blob content;
+        if( size<0 ){
+          xfer_fatal_error("invalid config record");
+          return ++nErr;
+        }
         blob_zero(&content);
         blob_extract(xfer.pIn, size, &content);
         g.perm.Admin = g.perm.RdAddr = 1;
@@ -2617,6 +2651,10 @@ int client_sync(
       */
       if( blob_eq(&xfer.aToken[0], "clone_seqno") && xfer.nToken==2 ){
         blob_is_int(&xfer.aToken[1], &cloneSeqno);
+        if( cloneSeqno<0 ){
+          xfer_fatal_error("invalid clone_seqno");
+          return ++nErr;
+        }
       }else
 
       /*   message MESSAGE
@@ -2694,6 +2732,10 @@ int client_sync(
           defossilize(zUser);
           iNow = time(NULL);
           if( blob_is_int64(&xfer.aToken[3], &mtime) && iNow>mtime ){
+            if( mtime<0 ){
+              xfer_fatal_error("invalid ci-lock-fail time");
+              return ++nErr;
+            }
             iNow = time(NULL);
             fossil_print("\nParent check-in locked by %s %s ago\n",
                zUser, human_readable_age((iNow+1-mtime)/86400.0));
@@ -2808,8 +2850,10 @@ int client_sync(
                    blob_size(&recv), nCardRcvd,
                    xfer.nFileRcvd, xfer.nDeltaRcvd + xfer.nDanglingFile);
     }else{
-      fossil_print(zBriefFormat /*works-like:"%d%d%d"*/,
-                   nRoundtrip, nArtifactSent, nArtifactRcvd);
+      if( bOutIsTty!=0 ){
+        fossil_print(zBriefFormat /*works-like:"%d%d%d"*/,
+                     nRoundtrip, nArtifactSent, nArtifactRcvd);
+      }
     }
     nUncRcvd += blob_size(&recv);
     blob_reset(&recv);
@@ -2867,7 +2911,11 @@ int client_sync(
                     db_timespan_name(-rSkew));
      g.clockSkewSeen = 1;
   }
-
+  if( bOutIsTty==0 ){
+    fossil_print(zBriefFormat /*works-like:"%d%d%d"*/,
+                 nRoundtrip, nArtifactSent, nArtifactRcvd);
+    fossil_force_newline();
+  }
   fossil_force_newline();
   if( g.zHttpCmd==0 ){
     if( syncFlags & SYNC_VERBOSE ){
